/****************************************************************************
** Meta object code from reading C++ file 'kcalc.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.0)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../kcalc-26.04.0/kcalc.h"
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'kcalc.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.0. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN7GeneralE_t {};
} // unnamed namespace

template <> constexpr inline auto General::qt_create_metaobjectdata<qt_meta_tag_ZN7GeneralE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "General"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<General, qt_meta_tag_ZN7GeneralE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject General::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7GeneralE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7GeneralE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN7GeneralE_t>.metaTypes,
    nullptr
} };

void General::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<General *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *General::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *General::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN7GeneralE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::General"))
        return static_cast< Ui::General*>(this);
    return QWidget::qt_metacast(_clname);
}

int General::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN5FontsE_t {};
} // unnamed namespace

template <> constexpr inline auto Fonts::qt_create_metaobjectdata<qt_meta_tag_ZN5FontsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Fonts"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Fonts, qt_meta_tag_ZN5FontsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Fonts::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5FontsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5FontsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN5FontsE_t>.metaTypes,
    nullptr
} };

void Fonts::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Fonts *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *Fonts::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Fonts::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN5FontsE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::Fonts"))
        return static_cast< Ui::Fonts*>(this);
    return QWidget::qt_metacast(_clname);
}

int Fonts::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN9ConstantsE_t {};
} // unnamed namespace

template <> constexpr inline auto Constants::qt_create_metaobjectdata<qt_meta_tag_ZN9ConstantsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Constants"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Constants, qt_meta_tag_ZN9ConstantsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Constants::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9ConstantsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9ConstantsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN9ConstantsE_t>.metaTypes,
    nullptr
} };

void Constants::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Constants *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *Constants::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Constants::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN9ConstantsE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::Constants"))
        return static_cast< Ui::Constants*>(this);
    return QWidget::qt_metacast(_clname);
}

int Constants::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN6ColorsE_t {};
} // unnamed namespace

template <> constexpr inline auto Colors::qt_create_metaobjectdata<qt_meta_tag_ZN6ColorsE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "Colors"
    };

    QtMocHelpers::UintData qt_methods {
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<Colors, qt_meta_tag_ZN6ColorsE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject Colors::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6ColorsE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6ColorsE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN6ColorsE_t>.metaTypes,
    nullptr
} };

void Colors::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<Colors *>(_o);
    (void)_t;
    (void)_c;
    (void)_id;
    (void)_a;
}

const QMetaObject *Colors::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Colors::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN6ColorsE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::Colors"))
        return static_cast< Ui::Colors*>(this);
    return QWidget::qt_metacast(_clname);
}

int Colors::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    return _id;
}
namespace {
struct qt_meta_tag_ZN11KCalculatorE_t {};
} // unnamed namespace

template <> constexpr inline auto KCalculator::qt_create_metaobjectdata<qt_meta_tag_ZN11KCalculatorE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "KCalculator",
        "switchMode",
        "",
        "ButtonModeFlags",
        "switchShowAccels",
        "changeButtonNames",
        "updateSettings",
        "setColors",
        "setFonts",
        "updateResultDisplay",
        "showSettings",
        "slotSetSimpleMode",
        "slotSetScienceMode",
        "slotSetStatisticMode",
        "slotSetNumeralMode",
        "slotHistoryshow",
        "toggled",
        "slotConstantsShow",
        "slotBitsetshow",
        "slotAngleSelected",
        "QAbstractButton*",
        "button",
        "checked",
        "slotBaseSelected",
        "slotNumberclicked",
        "slotEEclicked",
        "slotShifttoggled",
        "myboolean",
        "slotMemRecallclicked",
        "slotMemStoreclicked",
        "slotSinclicked",
        "slotPlusMinusclicked",
        "slotMemPlusMinusclicked",
        "slotCosclicked",
        "slotReciclicked",
        "slotTanclicked",
        "slotFactorialclicked",
        "slotLogclicked",
        "slotSquareclicked",
        "slotSqrtclicked",
        "slotLnclicked",
        "slotIclicked",
        "slotPowerclicked",
        "slotMemClearclicked",
        "slotClearclicked",
        "slotAllClearclicked",
        "slotParenOpenclicked",
        "slotParenCloseclicked",
        "slotANDclicked",
        "slotMultiplicationclicked",
        "slotDivisionclicked",
        "slotORclicked",
        "slotXORclicked",
        "slotPlusclicked",
        "slotMinusclicked",
        "slotLeftShiftclicked",
        "slotRightShiftclicked",
        "slotPeriodclicked",
        "slotEqualclicked",
        "slotPercentclicked",
        "slotNegateclicked",
        "slotModclicked",
        "slotStatNumclicked",
        "slotStatMeanclicked",
        "slotStatStdDevclicked",
        "slotStatMedianclicked",
        "slotStatDataInputclicked",
        "slotStatClearDataclicked",
        "slotHyptoggled",
        "flag",
        "slotConstclicked",
        "slotBackspaceclicked",
        "slotInputChanged",
        "slotClearResult",
        "slotConstantToDisplay",
        "ScienceConstant",
        "constChosen",
        "slotChooseScientificConst0",
        "slotChooseScientificConst1",
        "slotChooseScientificConst2",
        "slotChooseScientificConst3",
        "slotChooseScientificConst4",
        "slotChooseScientificConst5",
        "slotBitsetChanged",
        "value",
        "slotUpdateBitset",
        "KNumber",
        "slotBaseModeAmountChanged",
        "number",
        "slotClearBaseModeAmount",
        "slotPaste"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'switchMode'
        QtMocHelpers::SignalData<void(ButtonModeFlags, bool)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 3, 2 }, { QMetaType::Bool, 2 },
        }}),
        // Signal 'switchShowAccels'
        QtMocHelpers::SignalData<void(bool)>(4, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Bool, 2 },
        }}),
        // Slot 'changeButtonNames'
        QtMocHelpers::SlotData<void()>(5, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateSettings'
        QtMocHelpers::SlotData<void()>(6, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'setColors'
        QtMocHelpers::SlotData<void()>(7, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'setFonts'
        QtMocHelpers::SlotData<void()>(8, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'updateResultDisplay'
        QtMocHelpers::SlotData<void()>(9, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'showSettings'
        QtMocHelpers::SlotData<void()>(10, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSetSimpleMode'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSetScienceMode'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSetStatisticMode'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSetNumeralMode'
        QtMocHelpers::SlotData<void()>(14, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotHistoryshow'
        QtMocHelpers::SlotData<void(bool)>(15, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'slotConstantsShow'
        QtMocHelpers::SlotData<void(bool)>(17, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'slotBitsetshow'
        QtMocHelpers::SlotData<void(bool)>(18, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 16 },
        }}),
        // Slot 'slotAngleSelected'
        QtMocHelpers::SlotData<void(QAbstractButton *, bool)>(19, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 20, 21 }, { QMetaType::Bool, 22 },
        }}),
        // Slot 'slotBaseSelected'
        QtMocHelpers::SlotData<void()>(23, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotNumberclicked'
        QtMocHelpers::SlotData<void(QAbstractButton *)>(24, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 20, 21 },
        }}),
        // Slot 'slotEEclicked'
        QtMocHelpers::SlotData<void()>(25, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotShifttoggled'
        QtMocHelpers::SlotData<void(bool)>(26, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 27 },
        }}),
        // Slot 'slotMemRecallclicked'
        QtMocHelpers::SlotData<void()>(28, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotMemStoreclicked'
        QtMocHelpers::SlotData<void()>(29, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSinclicked'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotPlusMinusclicked'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotMemPlusMinusclicked'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotCosclicked'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotReciclicked'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotTanclicked'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotFactorialclicked'
        QtMocHelpers::SlotData<void()>(36, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotLogclicked'
        QtMocHelpers::SlotData<void()>(37, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSquareclicked'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotSqrtclicked'
        QtMocHelpers::SlotData<void()>(39, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotLnclicked'
        QtMocHelpers::SlotData<void()>(40, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotIclicked'
        QtMocHelpers::SlotData<void()>(41, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotPowerclicked'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotMemClearclicked'
        QtMocHelpers::SlotData<void()>(43, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotClearclicked'
        QtMocHelpers::SlotData<void()>(44, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotAllClearclicked'
        QtMocHelpers::SlotData<void()>(45, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotParenOpenclicked'
        QtMocHelpers::SlotData<void()>(46, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotParenCloseclicked'
        QtMocHelpers::SlotData<void()>(47, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotANDclicked'
        QtMocHelpers::SlotData<void()>(48, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotMultiplicationclicked'
        QtMocHelpers::SlotData<void()>(49, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotDivisionclicked'
        QtMocHelpers::SlotData<void()>(50, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotORclicked'
        QtMocHelpers::SlotData<void()>(51, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotXORclicked'
        QtMocHelpers::SlotData<void()>(52, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotPlusclicked'
        QtMocHelpers::SlotData<void()>(53, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotMinusclicked'
        QtMocHelpers::SlotData<void()>(54, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotLeftShiftclicked'
        QtMocHelpers::SlotData<void()>(55, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotRightShiftclicked'
        QtMocHelpers::SlotData<void()>(56, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotPeriodclicked'
        QtMocHelpers::SlotData<void()>(57, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotEqualclicked'
        QtMocHelpers::SlotData<void()>(58, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotPercentclicked'
        QtMocHelpers::SlotData<void()>(59, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotNegateclicked'
        QtMocHelpers::SlotData<void()>(60, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotModclicked'
        QtMocHelpers::SlotData<void()>(61, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotStatNumclicked'
        QtMocHelpers::SlotData<void()>(62, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotStatMeanclicked'
        QtMocHelpers::SlotData<void()>(63, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotStatStdDevclicked'
        QtMocHelpers::SlotData<void()>(64, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotStatMedianclicked'
        QtMocHelpers::SlotData<void()>(65, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotStatDataInputclicked'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotStatClearDataclicked'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotHyptoggled'
        QtMocHelpers::SlotData<void(bool)>(68, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Bool, 69 },
        }}),
        // Slot 'slotConstclicked'
        QtMocHelpers::SlotData<void(int)>(70, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::Int, 2 },
        }}),
        // Slot 'slotBackspaceclicked'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotInputChanged'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotClearResult'
        QtMocHelpers::SlotData<void()>(73, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotConstantToDisplay'
        QtMocHelpers::SlotData<void(const ScienceConstant &)>(74, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 75, 76 },
        }}),
        // Slot 'slotChooseScientificConst0'
        QtMocHelpers::SlotData<void(const ScienceConstant &)>(77, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 75, 2 },
        }}),
        // Slot 'slotChooseScientificConst1'
        QtMocHelpers::SlotData<void(const ScienceConstant &)>(78, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 75, 2 },
        }}),
        // Slot 'slotChooseScientificConst2'
        QtMocHelpers::SlotData<void(const ScienceConstant &)>(79, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 75, 2 },
        }}),
        // Slot 'slotChooseScientificConst3'
        QtMocHelpers::SlotData<void(const ScienceConstant &)>(80, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 75, 2 },
        }}),
        // Slot 'slotChooseScientificConst4'
        QtMocHelpers::SlotData<void(const ScienceConstant &)>(81, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 75, 2 },
        }}),
        // Slot 'slotChooseScientificConst5'
        QtMocHelpers::SlotData<void(const ScienceConstant &)>(82, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 75, 2 },
        }}),
        // Slot 'slotBitsetChanged'
        QtMocHelpers::SlotData<void(qint64)>(83, 2, QMC::AccessProtected, QMetaType::Void, {{
            { QMetaType::LongLong, 84 },
        }}),
        // Slot 'slotUpdateBitset'
        QtMocHelpers::SlotData<void(const KNumber &)>(85, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 86, 2 },
        }}),
        // Slot 'slotBaseModeAmountChanged'
        QtMocHelpers::SlotData<void(const KNumber &)>(87, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 86, 88 },
        }}),
        // Slot 'slotClearBaseModeAmount'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'slotPaste'
        QtMocHelpers::SlotData<void()>(90, 2, QMC::AccessProtected, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<KCalculator, qt_meta_tag_ZN11KCalculatorE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject KCalculator::staticMetaObject = { {
    QMetaObject::SuperData::link<KXmlGuiWindow::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KCalculatorE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KCalculatorE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN11KCalculatorE_t>.metaTypes,
    nullptr
} };

void KCalculator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<KCalculator *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->switchMode((*reinterpret_cast<std::add_pointer_t<ButtonModeFlags>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 1: _t->switchShowAccels((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 2: _t->changeButtonNames(); break;
        case 3: _t->updateSettings(); break;
        case 4: _t->setColors(); break;
        case 5: _t->setFonts(); break;
        case 6: _t->updateResultDisplay(); break;
        case 7: _t->showSettings(); break;
        case 8: _t->slotSetSimpleMode(); break;
        case 9: _t->slotSetScienceMode(); break;
        case 10: _t->slotSetStatisticMode(); break;
        case 11: _t->slotSetNumeralMode(); break;
        case 12: _t->slotHistoryshow((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 13: _t->slotConstantsShow((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 14: _t->slotBitsetshow((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 15: _t->slotAngleSelected((*reinterpret_cast<std::add_pointer_t<QAbstractButton*>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[2]))); break;
        case 16: _t->slotBaseSelected(); break;
        case 17: _t->slotNumberclicked((*reinterpret_cast<std::add_pointer_t<QAbstractButton*>>(_a[1]))); break;
        case 18: _t->slotEEclicked(); break;
        case 19: _t->slotShifttoggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 20: _t->slotMemRecallclicked(); break;
        case 21: _t->slotMemStoreclicked(); break;
        case 22: _t->slotSinclicked(); break;
        case 23: _t->slotPlusMinusclicked(); break;
        case 24: _t->slotMemPlusMinusclicked(); break;
        case 25: _t->slotCosclicked(); break;
        case 26: _t->slotReciclicked(); break;
        case 27: _t->slotTanclicked(); break;
        case 28: _t->slotFactorialclicked(); break;
        case 29: _t->slotLogclicked(); break;
        case 30: _t->slotSquareclicked(); break;
        case 31: _t->slotSqrtclicked(); break;
        case 32: _t->slotLnclicked(); break;
        case 33: _t->slotIclicked(); break;
        case 34: _t->slotPowerclicked(); break;
        case 35: _t->slotMemClearclicked(); break;
        case 36: _t->slotClearclicked(); break;
        case 37: _t->slotAllClearclicked(); break;
        case 38: _t->slotParenOpenclicked(); break;
        case 39: _t->slotParenCloseclicked(); break;
        case 40: _t->slotANDclicked(); break;
        case 41: _t->slotMultiplicationclicked(); break;
        case 42: _t->slotDivisionclicked(); break;
        case 43: _t->slotORclicked(); break;
        case 44: _t->slotXORclicked(); break;
        case 45: _t->slotPlusclicked(); break;
        case 46: _t->slotMinusclicked(); break;
        case 47: _t->slotLeftShiftclicked(); break;
        case 48: _t->slotRightShiftclicked(); break;
        case 49: _t->slotPeriodclicked(); break;
        case 50: _t->slotEqualclicked(); break;
        case 51: _t->slotPercentclicked(); break;
        case 52: _t->slotNegateclicked(); break;
        case 53: _t->slotModclicked(); break;
        case 54: _t->slotStatNumclicked(); break;
        case 55: _t->slotStatMeanclicked(); break;
        case 56: _t->slotStatStdDevclicked(); break;
        case 57: _t->slotStatMedianclicked(); break;
        case 58: _t->slotStatDataInputclicked(); break;
        case 59: _t->slotStatClearDataclicked(); break;
        case 60: _t->slotHyptoggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 61: _t->slotConstclicked((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 62: _t->slotBackspaceclicked(); break;
        case 63: _t->slotInputChanged(); break;
        case 64: _t->slotClearResult(); break;
        case 65: _t->slotConstantToDisplay((*reinterpret_cast<std::add_pointer_t<ScienceConstant>>(_a[1]))); break;
        case 66: _t->slotChooseScientificConst0((*reinterpret_cast<std::add_pointer_t<ScienceConstant>>(_a[1]))); break;
        case 67: _t->slotChooseScientificConst1((*reinterpret_cast<std::add_pointer_t<ScienceConstant>>(_a[1]))); break;
        case 68: _t->slotChooseScientificConst2((*reinterpret_cast<std::add_pointer_t<ScienceConstant>>(_a[1]))); break;
        case 69: _t->slotChooseScientificConst3((*reinterpret_cast<std::add_pointer_t<ScienceConstant>>(_a[1]))); break;
        case 70: _t->slotChooseScientificConst4((*reinterpret_cast<std::add_pointer_t<ScienceConstant>>(_a[1]))); break;
        case 71: _t->slotChooseScientificConst5((*reinterpret_cast<std::add_pointer_t<ScienceConstant>>(_a[1]))); break;
        case 72: _t->slotBitsetChanged((*reinterpret_cast<std::add_pointer_t<qint64>>(_a[1]))); break;
        case 73: _t->slotUpdateBitset((*reinterpret_cast<std::add_pointer_t<KNumber>>(_a[1]))); break;
        case 74: _t->slotBaseModeAmountChanged((*reinterpret_cast<std::add_pointer_t<KNumber>>(_a[1]))); break;
        case 75: _t->slotClearBaseModeAmount(); break;
        case 76: _t->slotPaste(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 15:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractButton* >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractButton* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (KCalculator::*)(ButtonModeFlags , bool )>(_a, &KCalculator::switchMode, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (KCalculator::*)(bool )>(_a, &KCalculator::switchShowAccels, 1))
            return;
    }
}

const QMetaObject *KCalculator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *KCalculator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN11KCalculatorE_t>.strings))
        return static_cast<void*>(this);
    return KXmlGuiWindow::qt_metacast(_clname);
}

int KCalculator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = KXmlGuiWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 77)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 77;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 77)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 77;
    }
    return _id;
}

// SIGNAL 0
void KCalculator::switchMode(ButtonModeFlags _t1, bool _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void KCalculator::switchShowAccels(bool _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1);
}
QT_WARNING_POP
