/********************************************************************************
** Form generated from reading UI file 'colors.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_COLORS_H
#define UI_COLORS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcolorbutton.h"

QT_BEGIN_NAMESPACE

class Ui_Colors
{
public:
    QVBoxLayout *vboxLayout;
    QCheckBox *kcfg_FollowSystemTheme;
    QGroupBox *displayColorsBox;
    QHBoxLayout *hboxLayout;
    QGridLayout *gridLayout;
    QLabel *textLabel1;
    KColorButton *kcfg_ForeColor;
    QLabel *textLabel2;
    KColorButton *kcfg_BackColor;
    QSpacerItem *spacerItem;
    QGroupBox *buttonColorsBox;
    QHBoxLayout *hboxLayout1;
    QGridLayout *gridLayout1;
    QLabel *textLabel6;
    QLabel *textLabel3;
    QLabel *textLabel5;
    KColorButton *kcfg_FunctionButtonsColor;
    KColorButton *kcfg_FunctionFontsColor;
    QLabel *textLabel4;
    KColorButton *kcfg_StatButtonsColor;
    KColorButton *kcfg_StatFontsColor;
    QLabel *textLabel9;
    KColorButton *kcfg_HexButtonsColor;
    KColorButton *kcfg_HexFontsColor;
    QLabel *textLabel11;
    KColorButton *kcfg_NumberButtonsColor;
    KColorButton *kcfg_NumberFontsColor;
    QLabel *textLabel10;
    KColorButton *kcfg_MemoryButtonsColor;
    KColorButton *kcfg_MemoryFontsColor;
    QLabel *textLabel12;
    KColorButton *kcfg_OperationButtonsColor;
    KColorButton *kcfg_OperationFontsColor;
    QLabel *textLabel13;
    KColorButton *kcfg_ConstantsButtonsColor;
    KColorButton *kcfg_ConstantsFontsColor;
    QSpacerItem *spacerItem1;
    QSpacerItem *spacerItem2;

    void setupUi(QWidget *Colors)
    {
        if (Colors->objectName().isEmpty())
            Colors->setObjectName("Colors");
        Colors->resize(389, 444);
        vboxLayout = new QVBoxLayout(Colors);
        vboxLayout->setObjectName("vboxLayout");
        kcfg_FollowSystemTheme = new QCheckBox(Colors);
        kcfg_FollowSystemTheme->setObjectName("kcfg_FollowSystemTheme");
        kcfg_FollowSystemTheme->setChecked(true);

        vboxLayout->addWidget(kcfg_FollowSystemTheme);

        displayColorsBox = new QGroupBox(Colors);
        displayColorsBox->setObjectName("displayColorsBox");
        displayColorsBox->setEnabled(false);
        hboxLayout = new QHBoxLayout(displayColorsBox);
        hboxLayout->setObjectName("hboxLayout");
        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        textLabel1 = new QLabel(displayColorsBox);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 0, 0, 1, 1);

        kcfg_ForeColor = new KColorButton(displayColorsBox);
        kcfg_ForeColor->setObjectName("kcfg_ForeColor");
        kcfg_ForeColor->setProperty("color", QVariant(QColor(0, 0, 0)));

        gridLayout->addWidget(kcfg_ForeColor, 0, 1, 1, 1);

        textLabel2 = new QLabel(displayColorsBox);
        textLabel2->setObjectName("textLabel2");
        textLabel2->setWordWrap(false);

        gridLayout->addWidget(textLabel2, 1, 0, 1, 1);

        kcfg_BackColor = new KColorButton(displayColorsBox);
        kcfg_BackColor->setObjectName("kcfg_BackColor");
        kcfg_BackColor->setProperty("color", QVariant(QColor(189, 255, 180)));

        gridLayout->addWidget(kcfg_BackColor, 1, 1, 1, 1);


        hboxLayout->addLayout(gridLayout);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);


        vboxLayout->addWidget(displayColorsBox);

        buttonColorsBox = new QGroupBox(Colors);
        buttonColorsBox->setObjectName("buttonColorsBox");
        buttonColorsBox->setEnabled(false);
        hboxLayout1 = new QHBoxLayout(buttonColorsBox);
        hboxLayout1->setObjectName("hboxLayout1");
        gridLayout1 = new QGridLayout();
        gridLayout1->setObjectName("gridLayout1");
        textLabel6 = new QLabel(buttonColorsBox);
        textLabel6->setObjectName("textLabel6");
        textLabel6->setWordWrap(false);

        gridLayout1->addWidget(textLabel6, 0, 1, 1, 1);

        textLabel3 = new QLabel(buttonColorsBox);
        textLabel3->setObjectName("textLabel3");
        textLabel3->setWordWrap(false);

        gridLayout1->addWidget(textLabel3, 0, 2, 1, 1);

        textLabel5 = new QLabel(buttonColorsBox);
        textLabel5->setObjectName("textLabel5");
        textLabel5->setWordWrap(false);

        gridLayout1->addWidget(textLabel5, 1, 0, 1, 1);

        kcfg_FunctionButtonsColor = new KColorButton(buttonColorsBox);
        kcfg_FunctionButtonsColor->setObjectName("kcfg_FunctionButtonsColor");

        gridLayout1->addWidget(kcfg_FunctionButtonsColor, 1, 1, 1, 1);

        kcfg_FunctionFontsColor = new KColorButton(buttonColorsBox);
        kcfg_FunctionFontsColor->setObjectName("kcfg_FunctionFontsColor");

        gridLayout1->addWidget(kcfg_FunctionFontsColor, 1, 2, 1, 1);

        textLabel4 = new QLabel(buttonColorsBox);
        textLabel4->setObjectName("textLabel4");
        textLabel4->setWordWrap(false);

        gridLayout1->addWidget(textLabel4, 2, 0, 1, 1);

        kcfg_StatButtonsColor = new KColorButton(buttonColorsBox);
        kcfg_StatButtonsColor->setObjectName("kcfg_StatButtonsColor");

        gridLayout1->addWidget(kcfg_StatButtonsColor, 2, 1, 1, 1);

        kcfg_StatFontsColor = new KColorButton(buttonColorsBox);
        kcfg_StatFontsColor->setObjectName("kcfg_StatFontsColor");

        gridLayout1->addWidget(kcfg_StatFontsColor, 2, 2, 1, 1);

        textLabel9 = new QLabel(buttonColorsBox);
        textLabel9->setObjectName("textLabel9");
        textLabel9->setWordWrap(false);

        gridLayout1->addWidget(textLabel9, 3, 0, 1, 1);

        kcfg_HexButtonsColor = new KColorButton(buttonColorsBox);
        kcfg_HexButtonsColor->setObjectName("kcfg_HexButtonsColor");

        gridLayout1->addWidget(kcfg_HexButtonsColor, 3, 1, 1, 1);

        kcfg_HexFontsColor = new KColorButton(buttonColorsBox);
        kcfg_HexFontsColor->setObjectName("kcfg_HexFontsColor");

        gridLayout1->addWidget(kcfg_HexFontsColor, 3, 2, 1, 1);

        textLabel11 = new QLabel(buttonColorsBox);
        textLabel11->setObjectName("textLabel11");
        textLabel11->setWordWrap(false);

        gridLayout1->addWidget(textLabel11, 4, 0, 1, 1);

        kcfg_NumberButtonsColor = new KColorButton(buttonColorsBox);
        kcfg_NumberButtonsColor->setObjectName("kcfg_NumberButtonsColor");

        gridLayout1->addWidget(kcfg_NumberButtonsColor, 4, 1, 1, 1);

        kcfg_NumberFontsColor = new KColorButton(buttonColorsBox);
        kcfg_NumberFontsColor->setObjectName("kcfg_NumberFontsColor");

        gridLayout1->addWidget(kcfg_NumberFontsColor, 4, 2, 1, 1);

        textLabel10 = new QLabel(buttonColorsBox);
        textLabel10->setObjectName("textLabel10");
        textLabel10->setWordWrap(false);

        gridLayout1->addWidget(textLabel10, 5, 0, 1, 1);

        kcfg_MemoryButtonsColor = new KColorButton(buttonColorsBox);
        kcfg_MemoryButtonsColor->setObjectName("kcfg_MemoryButtonsColor");

        gridLayout1->addWidget(kcfg_MemoryButtonsColor, 5, 1, 1, 1);

        kcfg_MemoryFontsColor = new KColorButton(buttonColorsBox);
        kcfg_MemoryFontsColor->setObjectName("kcfg_MemoryFontsColor");

        gridLayout1->addWidget(kcfg_MemoryFontsColor, 5, 2, 1, 1);

        textLabel12 = new QLabel(buttonColorsBox);
        textLabel12->setObjectName("textLabel12");
        textLabel12->setWordWrap(false);

        gridLayout1->addWidget(textLabel12, 6, 0, 1, 1);

        kcfg_OperationButtonsColor = new KColorButton(buttonColorsBox);
        kcfg_OperationButtonsColor->setObjectName("kcfg_OperationButtonsColor");

        gridLayout1->addWidget(kcfg_OperationButtonsColor, 6, 1, 1, 1);

        kcfg_OperationFontsColor = new KColorButton(buttonColorsBox);
        kcfg_OperationFontsColor->setObjectName("kcfg_OperationFontsColor");

        gridLayout1->addWidget(kcfg_OperationFontsColor, 6, 2, 1, 1);

        textLabel13 = new QLabel(buttonColorsBox);
        textLabel13->setObjectName("textLabel13");
        textLabel13->setWordWrap(false);

        gridLayout1->addWidget(textLabel13, 7, 0, 1, 1);

        kcfg_ConstantsButtonsColor = new KColorButton(buttonColorsBox);
        kcfg_ConstantsButtonsColor->setObjectName("kcfg_ConstantsButtonsColor");

        gridLayout1->addWidget(kcfg_ConstantsButtonsColor, 7, 1, 1, 1);

        kcfg_ConstantsFontsColor = new KColorButton(buttonColorsBox);
        kcfg_ConstantsFontsColor->setObjectName("kcfg_ConstantsFontsColor");

        gridLayout1->addWidget(kcfg_ConstantsFontsColor, 7, 2, 1, 1);


        hboxLayout1->addLayout(gridLayout1);

        spacerItem1 = new QSpacerItem(31, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(spacerItem1);


        vboxLayout->addWidget(buttonColorsBox);

        spacerItem2 = new QSpacerItem(151, 70, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(spacerItem2);

#if QT_CONFIG(shortcut)
        textLabel1->setBuddy(kcfg_ForeColor);
        textLabel2->setBuddy(kcfg_BackColor);
        textLabel5->setBuddy(kcfg_FunctionButtonsColor);
        textLabel4->setBuddy(kcfg_StatButtonsColor);
        textLabel9->setBuddy(kcfg_HexButtonsColor);
        textLabel11->setBuddy(kcfg_NumberButtonsColor);
        textLabel10->setBuddy(kcfg_MemoryButtonsColor);
        textLabel12->setBuddy(kcfg_OperationButtonsColor);
        textLabel13->setBuddy(kcfg_ConstantsButtonsColor);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(kcfg_FollowSystemTheme, kcfg_ForeColor);
        QWidget::setTabOrder(kcfg_ForeColor, kcfg_BackColor);
        QWidget::setTabOrder(kcfg_BackColor, kcfg_FunctionButtonsColor);
        QWidget::setTabOrder(kcfg_FunctionButtonsColor, kcfg_FunctionFontsColor);
        QWidget::setTabOrder(kcfg_FunctionFontsColor, kcfg_StatButtonsColor);
        QWidget::setTabOrder(kcfg_StatButtonsColor, kcfg_StatFontsColor);
        QWidget::setTabOrder(kcfg_StatFontsColor, kcfg_HexButtonsColor);
        QWidget::setTabOrder(kcfg_HexButtonsColor, kcfg_HexFontsColor);
        QWidget::setTabOrder(kcfg_HexFontsColor, kcfg_NumberButtonsColor);
        QWidget::setTabOrder(kcfg_NumberButtonsColor, kcfg_NumberFontsColor);
        QWidget::setTabOrder(kcfg_NumberFontsColor, kcfg_MemoryButtonsColor);
        QWidget::setTabOrder(kcfg_MemoryButtonsColor, kcfg_MemoryFontsColor);
        QWidget::setTabOrder(kcfg_MemoryFontsColor, kcfg_OperationButtonsColor);
        QWidget::setTabOrder(kcfg_OperationButtonsColor, kcfg_OperationFontsColor);

        retranslateUi(Colors);
        QObject::connect(kcfg_FollowSystemTheme, &QCheckBox::toggled, displayColorsBox, &QGroupBox::setDisabled);
        QObject::connect(kcfg_FollowSystemTheme, &QCheckBox::toggled, buttonColorsBox, &QGroupBox::setDisabled);

        QMetaObject::connectSlotsByName(Colors);
    } // setupUi

    void retranslateUi(QWidget *Colors)
    {
        kcfg_FollowSystemTheme->setText(tr2i18n("Follow system color scheme", nullptr));
        displayColorsBox->setTitle(tr2i18n("Display Colors", nullptr));
        textLabel1->setText(tr2i18n("&Foreground:", nullptr));
        kcfg_ForeColor->setText(QString());
        textLabel2->setText(tr2i18n("&Background:", nullptr));
        kcfg_BackColor->setText(QString());
        buttonColorsBox->setTitle(tr2i18n("Button Colors", nullptr));
        textLabel6->setText(tr2i18n("Background", nullptr));
        textLabel3->setText(tr2i18n("Foreground", nullptr));
        textLabel5->setText(tr2i18n("&Functions:", nullptr));
        kcfg_FunctionButtonsColor->setText(QString());
        kcfg_FunctionFontsColor->setText(QString());
        textLabel4->setText(tr2i18n("St&atistic functions:", nullptr));
        kcfg_StatButtonsColor->setText(QString());
        kcfg_StatFontsColor->setText(QString());
        textLabel9->setText(tr2i18n("He&xadecimals:", nullptr));
        kcfg_HexButtonsColor->setText(QString());
        kcfg_HexFontsColor->setText(QString());
        textLabel11->setText(tr2i18n("&Numbers:", nullptr));
        kcfg_NumberButtonsColor->setText(QString());
        kcfg_NumberFontsColor->setText(QString());
        textLabel10->setText(tr2i18n("&Memory:", nullptr));
        kcfg_MemoryButtonsColor->setText(QString());
        kcfg_MemoryFontsColor->setText(QString());
        textLabel12->setText(tr2i18n("O&perations:", nullptr));
        kcfg_OperationButtonsColor->setText(QString());
        kcfg_OperationFontsColor->setText(QString());
        textLabel13->setText(tr2i18n("&Constants:", nullptr));
        kcfg_ConstantsButtonsColor->setText(QString());
        kcfg_ConstantsFontsColor->setText(QString());
        (void)Colors;
    } // retranslateUi

};

namespace Ui {
    class Colors: public Ui_Colors {};
} // namespace Ui

QT_END_NAMESPACE

#endif // COLORS_H

