/********************************************************************************
** Form generated from reading UI file 'constants.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONSTANTS_H
#define UI_CONSTANTS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_Constants
{
public:
    QGridLayout *gridLayout;
    QLineEdit *kcfg_nameConstant0;
    QLineEdit *kcfg_valueConstant0;
    QPushButton *pushButton0;
    QLineEdit *kcfg_nameConstant1;
    QLineEdit *kcfg_valueConstant1;
    QPushButton *pushButton1;
    QLineEdit *kcfg_nameConstant2;
    QLineEdit *kcfg_valueConstant2;
    QPushButton *pushButton2;
    QLineEdit *kcfg_nameConstant3;
    QLineEdit *kcfg_valueConstant3;
    QPushButton *pushButton3;
    QLineEdit *kcfg_nameConstant4;
    QLineEdit *kcfg_valueConstant4;
    QPushButton *pushButton4;
    QLineEdit *kcfg_nameConstant5;
    QLineEdit *kcfg_valueConstant5;
    QPushButton *pushButton5;
    QSpacerItem *spacerItem;

    void setupUi(QWidget *Constants)
    {
        if (Constants->objectName().isEmpty())
            Constants->setObjectName("Constants");
        Constants->resize(185, 273);
        gridLayout = new QGridLayout(Constants);
        gridLayout->setObjectName("gridLayout");
        kcfg_nameConstant0 = new QLineEdit(Constants);
        kcfg_nameConstant0->setObjectName("kcfg_nameConstant0");
        kcfg_nameConstant0->setMaxLength(6);

        gridLayout->addWidget(kcfg_nameConstant0, 0, 0, 1, 1);

        kcfg_valueConstant0 = new QLineEdit(Constants);
        kcfg_valueConstant0->setObjectName("kcfg_valueConstant0");
        kcfg_valueConstant0->setMaxLength(40);

        gridLayout->addWidget(kcfg_valueConstant0, 0, 1, 1, 1);

        pushButton0 = new QPushButton(Constants);
        pushButton0->setObjectName("pushButton0");

        gridLayout->addWidget(pushButton0, 0, 2, 1, 1);

        kcfg_nameConstant1 = new QLineEdit(Constants);
        kcfg_nameConstant1->setObjectName("kcfg_nameConstant1");
        kcfg_nameConstant1->setMaxLength(6);

        gridLayout->addWidget(kcfg_nameConstant1, 1, 0, 1, 1);

        kcfg_valueConstant1 = new QLineEdit(Constants);
        kcfg_valueConstant1->setObjectName("kcfg_valueConstant1");
        kcfg_valueConstant1->setMaxLength(40);

        gridLayout->addWidget(kcfg_valueConstant1, 1, 1, 1, 1);

        pushButton1 = new QPushButton(Constants);
        pushButton1->setObjectName("pushButton1");

        gridLayout->addWidget(pushButton1, 1, 2, 1, 1);

        kcfg_nameConstant2 = new QLineEdit(Constants);
        kcfg_nameConstant2->setObjectName("kcfg_nameConstant2");
        kcfg_nameConstant2->setMaxLength(6);

        gridLayout->addWidget(kcfg_nameConstant2, 2, 0, 1, 1);

        kcfg_valueConstant2 = new QLineEdit(Constants);
        kcfg_valueConstant2->setObjectName("kcfg_valueConstant2");
        kcfg_valueConstant2->setMaxLength(40);

        gridLayout->addWidget(kcfg_valueConstant2, 2, 1, 1, 1);

        pushButton2 = new QPushButton(Constants);
        pushButton2->setObjectName("pushButton2");

        gridLayout->addWidget(pushButton2, 2, 2, 1, 1);

        kcfg_nameConstant3 = new QLineEdit(Constants);
        kcfg_nameConstant3->setObjectName("kcfg_nameConstant3");
        kcfg_nameConstant3->setMaxLength(6);

        gridLayout->addWidget(kcfg_nameConstant3, 3, 0, 1, 1);

        kcfg_valueConstant3 = new QLineEdit(Constants);
        kcfg_valueConstant3->setObjectName("kcfg_valueConstant3");
        kcfg_valueConstant3->setMaxLength(40);

        gridLayout->addWidget(kcfg_valueConstant3, 3, 1, 1, 1);

        pushButton3 = new QPushButton(Constants);
        pushButton3->setObjectName("pushButton3");

        gridLayout->addWidget(pushButton3, 3, 2, 1, 1);

        kcfg_nameConstant4 = new QLineEdit(Constants);
        kcfg_nameConstant4->setObjectName("kcfg_nameConstant4");
        kcfg_nameConstant4->setMaxLength(6);

        gridLayout->addWidget(kcfg_nameConstant4, 4, 0, 1, 1);

        kcfg_valueConstant4 = new QLineEdit(Constants);
        kcfg_valueConstant4->setObjectName("kcfg_valueConstant4");
        kcfg_valueConstant4->setMaxLength(40);

        gridLayout->addWidget(kcfg_valueConstant4, 4, 1, 1, 1);

        pushButton4 = new QPushButton(Constants);
        pushButton4->setObjectName("pushButton4");

        gridLayout->addWidget(pushButton4, 4, 2, 1, 1);

        kcfg_nameConstant5 = new QLineEdit(Constants);
        kcfg_nameConstant5->setObjectName("kcfg_nameConstant5");
        kcfg_nameConstant5->setMaxLength(6);

        gridLayout->addWidget(kcfg_nameConstant5, 5, 0, 1, 1);

        kcfg_valueConstant5 = new QLineEdit(Constants);
        kcfg_valueConstant5->setObjectName("kcfg_valueConstant5");
        kcfg_valueConstant5->setMaxLength(40);

        gridLayout->addWidget(kcfg_valueConstant5, 5, 1, 1, 1);

        pushButton5 = new QPushButton(Constants);
        pushButton5->setObjectName("pushButton5");

        gridLayout->addWidget(pushButton5, 5, 2, 1, 1);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        gridLayout->addItem(spacerItem, 6, 0, 1, 1);


        retranslateUi(Constants);

        QMetaObject::connectSlotsByName(Constants);
    } // setupUi

    void retranslateUi(QWidget *Constants)
    {
        Constants->setWindowTitle(tr2i18n("Constants", nullptr));
        pushButton0->setText(tr2i18n("Predefined", nullptr));
        pushButton1->setText(tr2i18n("Predefined", nullptr));
        pushButton2->setText(tr2i18n("Predefined", nullptr));
        pushButton3->setText(tr2i18n("Predefined", nullptr));
        pushButton4->setText(tr2i18n("Predefined", nullptr));
        pushButton5->setText(tr2i18n("Predefined", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Constants: public Ui_Constants {};
} // namespace Ui

QT_END_NAMESPACE

#endif // CONSTANTS_H

