/********************************************************************************
** Form generated from reading UI file 'kcalc.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_KCALC_H
#define UI_KCALC_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kcalc_bitset.h"
#include "kcalc_button.h"
#include "kcalc_const_button.h"
#include "kcalc_display.h"
#include "kcalc_display_frame.h"
#include "kcalc_history.h"
#include "kcalc_input_display.h"
#include "kcalc_numeralsystem_view.h"
#include "ksqueezedtextlabel.h"

QT_BEGIN_NAMESPACE

class Ui_KCalculator
{
public:
    QHBoxLayout *hboxLayout;
    QVBoxLayout *firstVerticalLayout;
    KCalcDisplayFrame *calc_display_frame;
    QVBoxLayout *DisplayLayout;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer_3;
    QLabel *shift_label;
    QSpacerItem *horizontalSpacer_4;
    QLabel *base_label;
    QSpacerItem *horizontalSpacer_5;
    QLabel *angle_units_label;
    QSpacerItem *horizontalSpacer_7;
    QLabel *memory_label;
    QSpacerItem *horizontalSpacer_6;
    KCalcInputDisplay *input_display;
    QSpacerItem *verticalSpacer;
    QFrame *line;
    KCalcDisplay *calc_display;
    QVBoxLayout *verticalLayout_5;
    QFrame *frame;
    QHBoxLayout *bitsetHelper;
    KCalcBitset *mBitset;
    QHBoxLayout *hboxLayout1;
    QRadioButton *degRadio;
    QRadioButton *radRadio;
    QRadioButton *gradRadio;
    QComboBox *numeralSystemComboBox;
    QSpacerItem *horizontalSpacer;
    KCalcNumeralSystemView *numeralSystemView;
    QHBoxLayout *hboxLayout2;
    QWidget *leftPad;
    QGridLayout *gridLayout;
    KCalcButton *pbSquare;
    KCalcButton *pbB;
    KCalcButton *pbOR;
    KCalcButton *pbE;
    KCalcButton *pbSin;
    KCalcConstButton *pbC2;
    KCalcButton *pbC;
    KCalcButton *pbEE;
    KCalcButton *pbMod;
    KCalcButton *pbNData;
    KCalcButton *pbLog;
    KCalcButton *pbTan;
    KCalcButton *pbF;
    KCalcButton *pbAND;
    KCalcConstButton *pbC6;
    KCalcButton *pbA;
    KCalcButton *pbD;
    KCalcConstButton *pbC1;
    KCalcButton *pbHyp;
    KCalcButton *pbXOR;
    KCalcButton *pbMean;
    KCalcButton *pbFactorial;
    KCalcButton *pbDat;
    KCalcConstButton *pbC3;
    KCalcButton *pbCmp;
    KCalcButton *pbCSt;
    KCalcButton *pbRsh;
    KCalcButton *pbCos;
    KCalcButton *pbReci;
    KCalcButton *pbLn;
    KCalcButton *pbI;
    KCalcButton *pbMed;
    KCalcConstButton *pbC5;
    KCalcConstButton *pbC4;
    KCalcButton *pbSd;
    KCalcButton *pbLsh;
    KCalcButton *pbSqrt;
    KCalcButton *pbPower;
    QWidget *numericPad;
    QGridLayout *gridLayout1;
    KCalcButton *pb6;
    KCalcButton *pb2;
    KCalcButton *pbPercent;
    KCalcButton *pb9;
    KCalcButton *pb4;
    KCalcButton *pb5;
    KCalcButton *pbMinus;
    KCalcButton *pbPlus;
    KCalcButton *pb1;
    KCalcButton *pbDivision;
    KCalcButton *pbMultiplication;
    KCalcButton *pb0;
    KCalcButton *pbPeriod;
    KCalcButton *pb3;
    KCalcButton *pbEqual;
    KCalcButton *pb7;
    KCalcButton *pb8;
    QWidget *rightPad;
    QGridLayout *gridLayout2;
    KCalcButton *pbAllClear;
    KCalcButton *pbMemClear;
    KCalcButton *pbMemRecall;
    KCalcButton *pbPlusMinus;
    KCalcButton *pbParenOpen;
    KCalcButton *pbClear;
    KCalcButton *pbMemPlusMinus;
    KCalcButton *pbMemStore;
    KCalcButton *pbParenClose;
    KCalcButton *pbBackspace;
    KCalcButton *pbShift;
    KCalcHistory *calc_history;

    void setupUi(QWidget *KCalculator)
    {
        if (KCalculator->objectName().isEmpty())
            KCalculator->setObjectName("KCalculator");
        KCalculator->resize(1346, 768);
        QSizePolicy sizePolicy(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(KCalculator->sizePolicy().hasHeightForWidth());
        KCalculator->setSizePolicy(sizePolicy);
        hboxLayout = new QHBoxLayout(KCalculator);
        hboxLayout->setSpacing(0);
        hboxLayout->setObjectName("hboxLayout");
        hboxLayout->setSizeConstraint(QLayout::SetMinimumSize);
        hboxLayout->setContentsMargins(0, 1, 0, 0);
        firstVerticalLayout = new QVBoxLayout();
        firstVerticalLayout->setSpacing(0);
        firstVerticalLayout->setObjectName("firstVerticalLayout");
        calc_display_frame = new KCalcDisplayFrame(KCalculator);
        calc_display_frame->setObjectName("calc_display_frame");
        calc_display_frame->setFrameShape(QFrame::StyledPanel);
        calc_display_frame->setFrameShadow(QFrame::Sunken);
        DisplayLayout = new QVBoxLayout(calc_display_frame);
        DisplayLayout->setSpacing(0);
        DisplayLayout->setObjectName("DisplayLayout");
        DisplayLayout->setSizeConstraint(QLayout::SetDefaultConstraint);
        DisplayLayout->setContentsMargins(10, 0, 10, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(0);
        horizontalLayout->setObjectName("horizontalLayout");
        horizontalLayout->setContentsMargins(-1, -1, -1, 0);
        horizontalSpacer_3 = new QSpacerItem(10, 0, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);

        shift_label = new QLabel(calc_display_frame);
        shift_label->setObjectName("shift_label");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(shift_label->sizePolicy().hasHeightForWidth());
        shift_label->setSizePolicy(sizePolicy1);
        shift_label->setMinimumSize(QSize(40, 0));
        QFont font;
        font.setPointSize(11);
        font.setBold(true);
        shift_label->setFont(font);
        shift_label->setFrameShape(QFrame::NoFrame);
        shift_label->setScaledContents(true);
        shift_label->setTextInteractionFlags(Qt::NoTextInteraction);

        horizontalLayout->addWidget(shift_label);

        horizontalSpacer_4 = new QSpacerItem(5, 0, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_4);

        base_label = new QLabel(calc_display_frame);
        base_label->setObjectName("base_label");
        sizePolicy1.setHeightForWidth(base_label->sizePolicy().hasHeightForWidth());
        base_label->setSizePolicy(sizePolicy1);
        base_label->setMinimumSize(QSize(35, 0));
        base_label->setFont(font);
        base_label->setFrameShape(QFrame::NoFrame);
        base_label->setScaledContents(true);

        horizontalLayout->addWidget(base_label);

        horizontalSpacer_5 = new QSpacerItem(5, 0, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_5);

        angle_units_label = new QLabel(calc_display_frame);
        angle_units_label->setObjectName("angle_units_label");
        sizePolicy1.setHeightForWidth(angle_units_label->sizePolicy().hasHeightForWidth());
        angle_units_label->setSizePolicy(sizePolicy1);
        angle_units_label->setMinimumSize(QSize(35, 0));
        angle_units_label->setFont(font);
        angle_units_label->setScaledContents(true);

        horizontalLayout->addWidget(angle_units_label);

        horizontalSpacer_7 = new QSpacerItem(5, 0, QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_7);

        memory_label = new QLabel(calc_display_frame);
        memory_label->setObjectName("memory_label");
        sizePolicy1.setHeightForWidth(memory_label->sizePolicy().hasHeightForWidth());
        memory_label->setSizePolicy(sizePolicy1);
        memory_label->setMinimumSize(QSize(20, 0));
        memory_label->setFont(font);
        memory_label->setFrameShape(QFrame::NoFrame);
        memory_label->setScaledContents(true);
        memory_label->setTextInteractionFlags(Qt::NoTextInteraction);

        horizontalLayout->addWidget(memory_label);

        horizontalSpacer_6 = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_6);

        horizontalLayout->setStretch(1, 1);
        horizontalLayout->setStretch(3, 1);
        horizontalLayout->setStretch(5, 1);
        horizontalLayout->setStretch(7, 1);
        horizontalLayout->setStretch(8, 28);

        DisplayLayout->addLayout(horizontalLayout);

        input_display = new KCalcInputDisplay(calc_display_frame);
        input_display->setObjectName("input_display");
        input_display->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        DisplayLayout->addWidget(input_display);

        verticalSpacer = new QSpacerItem(0, 5, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Fixed);

        DisplayLayout->addItem(verticalSpacer);

        line = new QFrame(calc_display_frame);
        line->setObjectName("line");
        line->setFrameShape(QFrame::Shape::HLine);
        line->setFrameShadow(QFrame::Shadow::Sunken);

        DisplayLayout->addWidget(line);

        calc_display = new KCalcDisplay(calc_display_frame);
        calc_display->setObjectName("calc_display");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Minimum, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(calc_display->sizePolicy().hasHeightForWidth());
        calc_display->setSizePolicy(sizePolicy2);
        calc_display->setMinimumSize(QSize(0, 60));
        calc_display->setMaximumSize(QSize(16777215, 16777215));
        calc_display->setFocusPolicy(Qt::StrongFocus);
        calc_display->setFrameShape(QFrame::StyledPanel);
        calc_display->setFrameShadow(QFrame::Sunken);
        verticalLayout_5 = new QVBoxLayout(calc_display);
        verticalLayout_5->setObjectName("verticalLayout_5");

        DisplayLayout->addWidget(calc_display);

        DisplayLayout->setStretch(0, 1);
        DisplayLayout->setStretch(1, 5);
        DisplayLayout->setStretch(4, 5);

        firstVerticalLayout->addWidget(calc_display_frame);

        frame = new QFrame(KCalculator);
        frame->setObjectName("frame");

        firstVerticalLayout->addWidget(frame);

        bitsetHelper = new QHBoxLayout();
        bitsetHelper->setObjectName("bitsetHelper");
        bitsetHelper->setContentsMargins(6, -1, 6, -1);
        mBitset = new KCalcBitset(KCalculator);
        mBitset->setObjectName("mBitset");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::MinimumExpanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(mBitset->sizePolicy().hasHeightForWidth());
        mBitset->setSizePolicy(sizePolicy3);

        bitsetHelper->addWidget(mBitset);


        firstVerticalLayout->addLayout(bitsetHelper);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName("hboxLayout1");
        hboxLayout1->setContentsMargins(6, 6, 6, 6);
        degRadio = new QRadioButton(KCalculator);
        degRadio->setObjectName("degRadio");

        hboxLayout1->addWidget(degRadio);

        radRadio = new QRadioButton(KCalculator);
        radRadio->setObjectName("radRadio");

        hboxLayout1->addWidget(radRadio);

        gradRadio = new QRadioButton(KCalculator);
        gradRadio->setObjectName("gradRadio");

        hboxLayout1->addWidget(gradRadio);

        numeralSystemComboBox = new QComboBox(KCalculator);
        numeralSystemComboBox->setObjectName("numeralSystemComboBox");

        hboxLayout1->addWidget(numeralSystemComboBox);

        horizontalSpacer = new QSpacerItem(0, 0, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout1->addItem(horizontalSpacer);

        numeralSystemView = new KCalcNumeralSystemView(KCalculator);
        numeralSystemView->setObjectName("numeralSystemView");

        hboxLayout1->addWidget(numeralSystemView);


        firstVerticalLayout->addLayout(hboxLayout1);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName("hboxLayout2");
        hboxLayout2->setContentsMargins(6, 0, 6, 6);
        leftPad = new QWidget(KCalculator);
        leftPad->setObjectName("leftPad");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy4.setHorizontalStretch(1);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(leftPad->sizePolicy().hasHeightForWidth());
        leftPad->setSizePolicy(sizePolicy4);
        gridLayout = new QGridLayout(leftPad);
        gridLayout->setSpacing(4);
        gridLayout->setObjectName("gridLayout");
        gridLayout->setContentsMargins(2, 2, 2, 2);
        pbSquare = new KCalcButton(leftPad);
        pbSquare->setObjectName("pbSquare");
        sizePolicy.setHeightForWidth(pbSquare->sizePolicy().hasHeightForWidth());
        pbSquare->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbSquare, 3, 3, 1, 1);

        pbB = new KCalcButton(leftPad);
        pbB->setObjectName("pbB");
        sizePolicy.setHeightForWidth(pbB->sizePolicy().hasHeightForWidth());
        pbB->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbB, 1, 4, 1, 1);

        pbOR = new KCalcButton(leftPad);
        pbOR->setObjectName("pbOR");
        sizePolicy.setHeightForWidth(pbOR->sizePolicy().hasHeightForWidth());
        pbOR->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbOR, 1, 2, 1, 1);

        pbE = new KCalcButton(leftPad);
        pbE->setObjectName("pbE");
        sizePolicy.setHeightForWidth(pbE->sizePolicy().hasHeightForWidth());
        pbE->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbE, 4, 4, 1, 1);

        pbSin = new KCalcButton(leftPad);
        pbSin->setObjectName("pbSin");
        sizePolicy.setHeightForWidth(pbSin->sizePolicy().hasHeightForWidth());
        pbSin->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbSin, 1, 1, 1, 1);

        pbC2 = new KCalcConstButton(leftPad);
        pbC2->setObjectName("pbC2");
        sizePolicy.setHeightForWidth(pbC2->sizePolicy().hasHeightForWidth());
        pbC2->setSizePolicy(sizePolicy);
#if QT_CONFIG(shortcut)
        pbC2->setShortcut(QString::fromUtf8("Alt+2"));
#endif // QT_CONFIG(shortcut)

        gridLayout->addWidget(pbC2, 1, 5, 1, 1);

        pbC = new KCalcButton(leftPad);
        pbC->setObjectName("pbC");
        sizePolicy.setHeightForWidth(pbC->sizePolicy().hasHeightForWidth());
        pbC->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbC, 2, 4, 1, 1);

        pbEE = new KCalcButton(leftPad);
        pbEE->setObjectName("pbEE");
        sizePolicy.setHeightForWidth(pbEE->sizePolicy().hasHeightForWidth());
        pbEE->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbEE, 8, 3, 1, 1);

        pbMod = new KCalcButton(leftPad);
        pbMod->setObjectName("pbMod");
        sizePolicy.setHeightForWidth(pbMod->sizePolicy().hasHeightForWidth());
        pbMod->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbMod, 0, 3, 1, 1);

        pbNData = new KCalcButton(leftPad);
        pbNData->setObjectName("pbNData");
        sizePolicy.setHeightForWidth(pbNData->sizePolicy().hasHeightForWidth());
        pbNData->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbNData, 0, 0, 1, 1);

        pbLog = new KCalcButton(leftPad);
        pbLog->setObjectName("pbLog");
        sizePolicy.setHeightForWidth(pbLog->sizePolicy().hasHeightForWidth());
        pbLog->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbLog, 4, 1, 1, 1);

        pbTan = new KCalcButton(leftPad);
        pbTan->setObjectName("pbTan");
        sizePolicy.setHeightForWidth(pbTan->sizePolicy().hasHeightForWidth());
        pbTan->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbTan, 3, 1, 1, 1);

        pbF = new KCalcButton(leftPad);
        pbF->setObjectName("pbF");
        sizePolicy.setHeightForWidth(pbF->sizePolicy().hasHeightForWidth());
        pbF->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbF, 7, 4, 1, 1);

        pbAND = new KCalcButton(leftPad);
        pbAND->setObjectName("pbAND");
        sizePolicy.setHeightForWidth(pbAND->sizePolicy().hasHeightForWidth());
        pbAND->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbAND, 0, 2, 1, 1);

        pbC6 = new KCalcConstButton(leftPad);
        pbC6->setObjectName("pbC6");
        sizePolicy.setHeightForWidth(pbC6->sizePolicy().hasHeightForWidth());
        pbC6->setSizePolicy(sizePolicy);
#if QT_CONFIG(shortcut)
        pbC6->setShortcut(QString::fromUtf8("Alt+6"));
#endif // QT_CONFIG(shortcut)

        gridLayout->addWidget(pbC6, 7, 5, 1, 1);

        pbA = new KCalcButton(leftPad);
        pbA->setObjectName("pbA");
        sizePolicy.setHeightForWidth(pbA->sizePolicy().hasHeightForWidth());
        pbA->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbA, 0, 4, 1, 1);

        pbD = new KCalcButton(leftPad);
        pbD->setObjectName("pbD");
        sizePolicy.setHeightForWidth(pbD->sizePolicy().hasHeightForWidth());
        pbD->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbD, 3, 4, 1, 1);

        pbC1 = new KCalcConstButton(leftPad);
        pbC1->setObjectName("pbC1");
        sizePolicy.setHeightForWidth(pbC1->sizePolicy().hasHeightForWidth());
        pbC1->setSizePolicy(sizePolicy);
#if QT_CONFIG(shortcut)
        pbC1->setShortcut(QString::fromUtf8("Alt+1"));
#endif // QT_CONFIG(shortcut)

        gridLayout->addWidget(pbC1, 0, 5, 1, 1);

        pbHyp = new KCalcButton(leftPad);
        pbHyp->setObjectName("pbHyp");
        sizePolicy.setHeightForWidth(pbHyp->sizePolicy().hasHeightForWidth());
        pbHyp->setSizePolicy(sizePolicy);
        pbHyp->setCheckable(true);

        gridLayout->addWidget(pbHyp, 0, 1, 1, 1);

        pbXOR = new KCalcButton(leftPad);
        pbXOR->setObjectName("pbXOR");
        sizePolicy.setHeightForWidth(pbXOR->sizePolicy().hasHeightForWidth());
        pbXOR->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbXOR, 2, 2, 1, 1);

        pbMean = new KCalcButton(leftPad);
        pbMean->setObjectName("pbMean");
        sizePolicy.setHeightForWidth(pbMean->sizePolicy().hasHeightForWidth());
        pbMean->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbMean, 1, 0, 1, 1);

        pbFactorial = new KCalcButton(leftPad);
        pbFactorial->setObjectName("pbFactorial");
        sizePolicy.setHeightForWidth(pbFactorial->sizePolicy().hasHeightForWidth());
        pbFactorial->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbFactorial, 2, 3, 1, 1);

        pbDat = new KCalcButton(leftPad);
        pbDat->setObjectName("pbDat");
        sizePolicy.setHeightForWidth(pbDat->sizePolicy().hasHeightForWidth());
        pbDat->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbDat, 4, 0, 1, 1);

        pbC3 = new KCalcConstButton(leftPad);
        pbC3->setObjectName("pbC3");
        sizePolicy.setHeightForWidth(pbC3->sizePolicy().hasHeightForWidth());
        pbC3->setSizePolicy(sizePolicy);
#if QT_CONFIG(shortcut)
        pbC3->setShortcut(QString::fromUtf8("Alt+3"));
#endif // QT_CONFIG(shortcut)

        gridLayout->addWidget(pbC3, 2, 5, 1, 1);

        pbCmp = new KCalcButton(leftPad);
        pbCmp->setObjectName("pbCmp");
        sizePolicy.setHeightForWidth(pbCmp->sizePolicy().hasHeightForWidth());
        pbCmp->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbCmp, 7, 2, 1, 1);

        pbCSt = new KCalcButton(leftPad);
        pbCSt->setObjectName("pbCSt");
        sizePolicy.setHeightForWidth(pbCSt->sizePolicy().hasHeightForWidth());
        pbCSt->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbCSt, 7, 0, 1, 1);

        pbRsh = new KCalcButton(leftPad);
        pbRsh->setObjectName("pbRsh");
        sizePolicy.setHeightForWidth(pbRsh->sizePolicy().hasHeightForWidth());
        pbRsh->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbRsh, 4, 2, 1, 1);

        pbCos = new KCalcButton(leftPad);
        pbCos->setObjectName("pbCos");
        sizePolicy.setHeightForWidth(pbCos->sizePolicy().hasHeightForWidth());
        pbCos->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbCos, 2, 1, 1, 1);

        pbReci = new KCalcButton(leftPad);
        pbReci->setObjectName("pbReci");
        sizePolicy.setHeightForWidth(pbReci->sizePolicy().hasHeightForWidth());
        pbReci->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbReci, 1, 3, 1, 1);

        pbLn = new KCalcButton(leftPad);
        pbLn->setObjectName("pbLn");
        sizePolicy.setHeightForWidth(pbLn->sizePolicy().hasHeightForWidth());
        pbLn->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbLn, 7, 1, 1, 1);

        pbI = new KCalcButton(leftPad);
        pbI->setObjectName("pbI");
        sizePolicy.setHeightForWidth(pbI->sizePolicy().hasHeightForWidth());
        pbI->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbI, 8, 1, 1, 1);

        pbMed = new KCalcButton(leftPad);
        pbMed->setObjectName("pbMed");
        sizePolicy.setHeightForWidth(pbMed->sizePolicy().hasHeightForWidth());
        pbMed->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbMed, 3, 0, 1, 1);

        pbC5 = new KCalcConstButton(leftPad);
        pbC5->setObjectName("pbC5");
        sizePolicy.setHeightForWidth(pbC5->sizePolicy().hasHeightForWidth());
        pbC5->setSizePolicy(sizePolicy);
#if QT_CONFIG(shortcut)
        pbC5->setShortcut(QString::fromUtf8("Alt+5"));
#endif // QT_CONFIG(shortcut)

        gridLayout->addWidget(pbC5, 4, 5, 1, 1);

        pbC4 = new KCalcConstButton(leftPad);
        pbC4->setObjectName("pbC4");
        sizePolicy.setHeightForWidth(pbC4->sizePolicy().hasHeightForWidth());
        pbC4->setSizePolicy(sizePolicy);
#if QT_CONFIG(shortcut)
        pbC4->setShortcut(QString::fromUtf8("Alt+4"));
#endif // QT_CONFIG(shortcut)

        gridLayout->addWidget(pbC4, 3, 5, 1, 1);

        pbSd = new KCalcButton(leftPad);
        pbSd->setObjectName("pbSd");
        sizePolicy.setHeightForWidth(pbSd->sizePolicy().hasHeightForWidth());
        pbSd->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbSd, 2, 0, 1, 1);

        pbLsh = new KCalcButton(leftPad);
        pbLsh->setObjectName("pbLsh");
        sizePolicy.setHeightForWidth(pbLsh->sizePolicy().hasHeightForWidth());
        pbLsh->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbLsh, 3, 2, 1, 1);

        pbSqrt = new KCalcButton(leftPad);
        pbSqrt->setObjectName("pbSqrt");
        sizePolicy.setHeightForWidth(pbSqrt->sizePolicy().hasHeightForWidth());
        pbSqrt->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbSqrt, 4, 3, 1, 1);

        pbPower = new KCalcButton(leftPad);
        pbPower->setObjectName("pbPower");
        sizePolicy.setHeightForWidth(pbPower->sizePolicy().hasHeightForWidth());
        pbPower->setSizePolicy(sizePolicy);

        gridLayout->addWidget(pbPower, 7, 3, 1, 1);


        hboxLayout2->addWidget(leftPad);

        numericPad = new QWidget(KCalculator);
        numericPad->setObjectName("numericPad");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Preferred);
        sizePolicy5.setHorizontalStretch(2);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(numericPad->sizePolicy().hasHeightForWidth());
        numericPad->setSizePolicy(sizePolicy5);
        gridLayout1 = new QGridLayout(numericPad);
        gridLayout1->setSpacing(4);
        gridLayout1->setObjectName("gridLayout1");
        gridLayout1->setContentsMargins(2, 2, 2, 2);
        pb6 = new KCalcButton(numericPad);
        pb6->setObjectName("pb6");
        sizePolicy.setHeightForWidth(pb6->sizePolicy().hasHeightForWidth());
        pb6->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(pb6, 2, 3, 1, 1);

        pb2 = new KCalcButton(numericPad);
        pb2->setObjectName("pb2");
        sizePolicy.setHeightForWidth(pb2->sizePolicy().hasHeightForWidth());
        pb2->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(pb2, 3, 1, 1, 1);

        pbPercent = new KCalcButton(numericPad);
        pbPercent->setObjectName("pbPercent");
        sizePolicy.setHeightForWidth(pbPercent->sizePolicy().hasHeightForWidth());
        pbPercent->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(pbPercent, 0, 0, 1, 1);

        pb9 = new KCalcButton(numericPad);
        pb9->setObjectName("pb9");
        sizePolicy.setHeightForWidth(pb9->sizePolicy().hasHeightForWidth());
        pb9->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(pb9, 1, 3, 1, 1);

        pb4 = new KCalcButton(numericPad);
        pb4->setObjectName("pb4");
        sizePolicy.setHeightForWidth(pb4->sizePolicy().hasHeightForWidth());
        pb4->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(pb4, 2, 0, 1, 1);

        pb5 = new KCalcButton(numericPad);
        pb5->setObjectName("pb5");
        sizePolicy.setHeightForWidth(pb5->sizePolicy().hasHeightForWidth());
        pb5->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(pb5, 2, 1, 1, 1);

        pbMinus = new KCalcButton(numericPad);
        pbMinus->setObjectName("pbMinus");
        sizePolicy.setHeightForWidth(pbMinus->sizePolicy().hasHeightForWidth());
        pbMinus->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(pbMinus, 0, 4, 1, 1);

        pbPlus = new KCalcButton(numericPad);
        pbPlus->setObjectName("pbPlus");
        sizePolicy.setHeightForWidth(pbPlus->sizePolicy().hasHeightForWidth());
        pbPlus->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(pbPlus, 1, 4, 2, 1);

        pb1 = new KCalcButton(numericPad);
        pb1->setObjectName("pb1");
        sizePolicy.setHeightForWidth(pb1->sizePolicy().hasHeightForWidth());
        pb1->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(pb1, 3, 0, 1, 1);

        pbDivision = new KCalcButton(numericPad);
        pbDivision->setObjectName("pbDivision");
        sizePolicy.setHeightForWidth(pbDivision->sizePolicy().hasHeightForWidth());
        pbDivision->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(pbDivision, 0, 1, 1, 1);

        pbMultiplication = new KCalcButton(numericPad);
        pbMultiplication->setObjectName("pbMultiplication");
        sizePolicy.setHeightForWidth(pbMultiplication->sizePolicy().hasHeightForWidth());
        pbMultiplication->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(pbMultiplication, 0, 3, 1, 1);

        pb0 = new KCalcButton(numericPad);
        pb0->setObjectName("pb0");
        sizePolicy.setHeightForWidth(pb0->sizePolicy().hasHeightForWidth());
        pb0->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(pb0, 4, 0, 1, 2);

        pbPeriod = new KCalcButton(numericPad);
        pbPeriod->setObjectName("pbPeriod");
        sizePolicy.setHeightForWidth(pbPeriod->sizePolicy().hasHeightForWidth());
        pbPeriod->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(pbPeriod, 4, 3, 1, 1);

        pb3 = new KCalcButton(numericPad);
        pb3->setObjectName("pb3");
        sizePolicy.setHeightForWidth(pb3->sizePolicy().hasHeightForWidth());
        pb3->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(pb3, 3, 3, 1, 1);

        pbEqual = new KCalcButton(numericPad);
        pbEqual->setObjectName("pbEqual");
        sizePolicy.setHeightForWidth(pbEqual->sizePolicy().hasHeightForWidth());
        pbEqual->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(pbEqual, 3, 4, 2, 1);

        pb7 = new KCalcButton(numericPad);
        pb7->setObjectName("pb7");
        sizePolicy.setHeightForWidth(pb7->sizePolicy().hasHeightForWidth());
        pb7->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(pb7, 1, 0, 1, 1);

        pb8 = new KCalcButton(numericPad);
        pb8->setObjectName("pb8");
        sizePolicy.setHeightForWidth(pb8->sizePolicy().hasHeightForWidth());
        pb8->setSizePolicy(sizePolicy);

        gridLayout1->addWidget(pb8, 1, 1, 1, 1);


        hboxLayout2->addWidget(numericPad);

        rightPad = new QWidget(KCalculator);
        rightPad->setObjectName("rightPad");
        sizePolicy4.setHeightForWidth(rightPad->sizePolicy().hasHeightForWidth());
        rightPad->setSizePolicy(sizePolicy4);
        gridLayout2 = new QGridLayout(rightPad);
        gridLayout2->setSpacing(4);
        gridLayout2->setObjectName("gridLayout2");
        gridLayout2->setContentsMargins(2, 2, 0, 2);
        pbAllClear = new KCalcButton(rightPad);
        pbAllClear->setObjectName("pbAllClear");
        sizePolicy.setHeightForWidth(pbAllClear->sizePolicy().hasHeightForWidth());
        pbAllClear->setSizePolicy(sizePolicy);

        gridLayout2->addWidget(pbAllClear, 2, 0, 1, 1);

        pbMemClear = new KCalcButton(rightPad);
        pbMemClear->setObjectName("pbMemClear");
        sizePolicy.setHeightForWidth(pbMemClear->sizePolicy().hasHeightForWidth());
        pbMemClear->setSizePolicy(sizePolicy);

        gridLayout2->addWidget(pbMemClear, 3, 1, 1, 1);

        pbMemRecall = new KCalcButton(rightPad);
        pbMemRecall->setObjectName("pbMemRecall");
        sizePolicy.setHeightForWidth(pbMemRecall->sizePolicy().hasHeightForWidth());
        pbMemRecall->setSizePolicy(sizePolicy);

        gridLayout2->addWidget(pbMemRecall, 4, 1, 1, 1);

        pbPlusMinus = new KCalcButton(rightPad);
        pbPlusMinus->setObjectName("pbPlusMinus");
        sizePolicy.setHeightForWidth(pbPlusMinus->sizePolicy().hasHeightForWidth());
        pbPlusMinus->setSizePolicy(sizePolicy);

        gridLayout2->addWidget(pbPlusMinus, 5, 0, 1, 1);

        pbParenOpen = new KCalcButton(rightPad);
        pbParenOpen->setObjectName("pbParenOpen");
        sizePolicy.setHeightForWidth(pbParenOpen->sizePolicy().hasHeightForWidth());
        pbParenOpen->setSizePolicy(sizePolicy);

        gridLayout2->addWidget(pbParenOpen, 3, 0, 1, 1);

        pbClear = new KCalcButton(rightPad);
        pbClear->setObjectName("pbClear");
        sizePolicy.setHeightForWidth(pbClear->sizePolicy().hasHeightForWidth());
        pbClear->setSizePolicy(sizePolicy);

        gridLayout2->addWidget(pbClear, 1, 0, 1, 1);

        pbMemPlusMinus = new KCalcButton(rightPad);
        pbMemPlusMinus->setObjectName("pbMemPlusMinus");
        sizePolicy.setHeightForWidth(pbMemPlusMinus->sizePolicy().hasHeightForWidth());
        pbMemPlusMinus->setSizePolicy(sizePolicy);

        gridLayout2->addWidget(pbMemPlusMinus, 5, 1, 1, 1);

        pbMemStore = new KCalcButton(rightPad);
        pbMemStore->setObjectName("pbMemStore");
        sizePolicy.setHeightForWidth(pbMemStore->sizePolicy().hasHeightForWidth());
        pbMemStore->setSizePolicy(sizePolicy);

        gridLayout2->addWidget(pbMemStore, 2, 1, 1, 1);

        pbParenClose = new KCalcButton(rightPad);
        pbParenClose->setObjectName("pbParenClose");
        sizePolicy.setHeightForWidth(pbParenClose->sizePolicy().hasHeightForWidth());
        pbParenClose->setSizePolicy(sizePolicy);

        gridLayout2->addWidget(pbParenClose, 4, 0, 1, 1);

        pbBackspace = new KCalcButton(rightPad);
        pbBackspace->setObjectName("pbBackspace");
        sizePolicy.setHeightForWidth(pbBackspace->sizePolicy().hasHeightForWidth());
        pbBackspace->setSizePolicy(sizePolicy);

        gridLayout2->addWidget(pbBackspace, 1, 1, 1, 1);

        pbShift = new KCalcButton(rightPad);
        pbShift->setObjectName("pbShift");
        sizePolicy.setHeightForWidth(pbShift->sizePolicy().hasHeightForWidth());
        pbShift->setSizePolicy(sizePolicy);
        pbShift->setCheckable(true);

        gridLayout2->addWidget(pbShift, 0, 0, 1, 2);


        hboxLayout2->addWidget(rightPad);


        firstVerticalLayout->addLayout(hboxLayout2);

        firstVerticalLayout->setStretch(0, 2);
        firstVerticalLayout->setStretch(4, 6);

        hboxLayout->addLayout(firstVerticalLayout);

        calc_history = new KCalcHistory(KCalculator);
        calc_history->setObjectName("calc_history");
        sizePolicy3.setHeightForWidth(calc_history->sizePolicy().hasHeightForWidth());
        calc_history->setSizePolicy(sizePolicy3);
        calc_history->setMinimumSize(QSize(256, 0));
        calc_history->setFocusPolicy(Qt::WheelFocus);

        hboxLayout->addWidget(calc_history);

        hboxLayout->setStretch(0, 2);
        hboxLayout->setStretch(1, 1);
        QWidget::setTabOrder(pb0, pb1);
        QWidget::setTabOrder(pb1, pb2);
        QWidget::setTabOrder(pb2, pb3);
        QWidget::setTabOrder(pb3, pb4);
        QWidget::setTabOrder(pb4, pb5);
        QWidget::setTabOrder(pb5, pb6);
        QWidget::setTabOrder(pb6, pb7);
        QWidget::setTabOrder(pb7, pb8);
        QWidget::setTabOrder(pb8, pb9);
        QWidget::setTabOrder(pb9, pbPeriod);
        QWidget::setTabOrder(pbPeriod, pbDivision);
        QWidget::setTabOrder(pbDivision, pbMultiplication);
        QWidget::setTabOrder(pbMultiplication, pbMinus);
        QWidget::setTabOrder(pbMinus, pbPlus);
        QWidget::setTabOrder(pbPlus, pbEqual);
        QWidget::setTabOrder(pbEqual, pbPercent);
        QWidget::setTabOrder(pbPercent, pbClear);
        QWidget::setTabOrder(pbClear, pbAllClear);
        QWidget::setTabOrder(pbAllClear, pbParenOpen);
        QWidget::setTabOrder(pbParenOpen, pbParenClose);
        QWidget::setTabOrder(pbParenClose, pbPlusMinus);
        QWidget::setTabOrder(pbPlusMinus, pbMemStore);
        QWidget::setTabOrder(pbMemStore, pbMemClear);
        QWidget::setTabOrder(pbMemClear, pbMemRecall);
        QWidget::setTabOrder(pbMemRecall, pbMemPlusMinus);
        QWidget::setTabOrder(pbMemPlusMinus, pbHyp);
        QWidget::setTabOrder(pbHyp, pbSin);
        QWidget::setTabOrder(pbSin, pbCos);
        QWidget::setTabOrder(pbCos, pbTan);
        QWidget::setTabOrder(pbTan, pbLog);
        QWidget::setTabOrder(pbLog, pbLn);
        QWidget::setTabOrder(pbLn, pbMod);
        QWidget::setTabOrder(pbMod, pbReci);
        QWidget::setTabOrder(pbReci, pbFactorial);
        QWidget::setTabOrder(pbFactorial, pbSquare);
        QWidget::setTabOrder(pbSquare, pbEE);
        QWidget::setTabOrder(pbEE, pbAND);
        QWidget::setTabOrder(pbAND, pbOR);
        QWidget::setTabOrder(pbOR, pbXOR);
        QWidget::setTabOrder(pbXOR, pbLsh);
        QWidget::setTabOrder(pbLsh, pbRsh);
        QWidget::setTabOrder(pbRsh, pbCmp);
        QWidget::setTabOrder(pbCmp, pbA);
        QWidget::setTabOrder(pbA, pbB);
        QWidget::setTabOrder(pbB, pbC);
        QWidget::setTabOrder(pbC, pbD);
        QWidget::setTabOrder(pbD, pbE);
        QWidget::setTabOrder(pbE, pbF);
        QWidget::setTabOrder(pbF, pbC1);
        QWidget::setTabOrder(pbC1, pbC2);
        QWidget::setTabOrder(pbC2, pbC3);
        QWidget::setTabOrder(pbC3, pbC4);
        QWidget::setTabOrder(pbC4, pbC5);
        QWidget::setTabOrder(pbC5, pbC6);
        QWidget::setTabOrder(pbC6, pbNData);
        QWidget::setTabOrder(pbNData, pbMean);
        QWidget::setTabOrder(pbMean, pbSd);
        QWidget::setTabOrder(pbSd, pbMed);
        QWidget::setTabOrder(pbMed, pbDat);
        QWidget::setTabOrder(pbDat, pbCSt);
        QWidget::setTabOrder(pbCSt, degRadio);
        QWidget::setTabOrder(degRadio, radRadio);
        QWidget::setTabOrder(radRadio, gradRadio);
        QWidget::setTabOrder(gradRadio, calc_history);

        retranslateUi(KCalculator);

        QMetaObject::connectSlotsByName(KCalculator);
    } // setupUi

    void retranslateUi(QWidget *KCalculator)
    {
        KCalculator->setWindowTitle(tr2i18n("KCalc", nullptr));
        shift_label->setText(QString());
        base_label->setText(tr2i18n("Dec", nullptr));
        angle_units_label->setText(QString());
        memory_label->setText(QString());
#if QT_CONFIG(accessibility)
        calc_display->setAccessibleDescription(tr2i18n("Result Display", nullptr));
#endif // QT_CONFIG(accessibility)
#if QT_CONFIG(tooltip)
        mBitset->setToolTip(tr2i18n("Click on a Bit to toggle it.", nullptr));
#endif // QT_CONFIG(tooltip)
        degRadio->setText(tr2i18n("&Deg", nullptr));
        radRadio->setText(tr2i18n("&Rad", nullptr));
        gradRadio->setText(tr2i18n("&Grad", nullptr));
#if QT_CONFIG(tooltip)
        numeralSystemComboBox->setToolTip(tr2i18n("Switch base", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pbSquare->setToolTip(tr2i18n("Square", nullptr));
#endif // QT_CONFIG(tooltip)
        pbSquare->setText(tr2i18n("x2", nullptr));
        pbB->setText(tr2i18n("B", nullptr));
#if QT_CONFIG(shortcut)
        pbB->setShortcut(tr2i18n("B", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        pbOR->setToolTip(tr2i18n("Bitwise OR", nullptr));
#endif // QT_CONFIG(tooltip)
        pbOR->setText(tr2i18n("OR", nullptr));
        pbE->setText(tr2i18n("E", nullptr));
#if QT_CONFIG(shortcut)
        pbE->setShortcut(tr2i18n("E", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        pbSin->setToolTip(tr2i18n("Sine", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(accessibility)
        pbSin->setAccessibleName(tr2i18n("Sine", nullptr));
#endif // QT_CONFIG(accessibility)
        pbSin->setText(tr2i18n("Sin", nullptr));
#if QT_CONFIG(shortcut)
        pbSin->setShortcut(tr2i18n("S", nullptr));
#endif // QT_CONFIG(shortcut)
        pbC2->setText(tr2i18n("C2", nullptr));
        pbC->setText(tr2i18n("C", nullptr));
#if QT_CONFIG(shortcut)
        pbC->setShortcut(tr2i18n("C", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        pbEE->setToolTip(tr2i18n("Exponent", nullptr));
#endif // QT_CONFIG(tooltip)
        pbEE->setText(tr2i18n("EXP", nullptr));
#if QT_CONFIG(shortcut)
        pbEE->setShortcut(tr2i18n("E", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        pbMod->setToolTip(tr2i18n("Modulo", nullptr));
#endif // QT_CONFIG(tooltip)
        pbMod->setText(tr2i18n("Mod", nullptr));
#if QT_CONFIG(tooltip)
        pbNData->setToolTip(tr2i18n("Number of data entered", nullptr));
#endif // QT_CONFIG(tooltip)
        pbNData->setText(tr2i18n("N", nullptr));
#if QT_CONFIG(tooltip)
        pbLog->setToolTip(tr2i18n("Logarithm to base 10", nullptr));
#endif // QT_CONFIG(tooltip)
        pbLog->setText(tr2i18n("Log", nullptr));
#if QT_CONFIG(shortcut)
        pbLog->setShortcut(tr2i18n("L", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        pbTan->setToolTip(tr2i18n("Tangent", nullptr));
#endif // QT_CONFIG(tooltip)
        pbTan->setText(tr2i18n("Tan", nullptr));
#if QT_CONFIG(shortcut)
        pbTan->setShortcut(tr2i18n("T", nullptr));
#endif // QT_CONFIG(shortcut)
        pbF->setText(tr2i18n("F", nullptr));
#if QT_CONFIG(shortcut)
        pbF->setShortcut(tr2i18n("F", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        pbAND->setToolTip(tr2i18n("Bitwise AND", nullptr));
#endif // QT_CONFIG(tooltip)
        pbAND->setText(tr2i18n("AND", nullptr));
        pbC6->setText(tr2i18n("C6", nullptr));
        pbA->setText(tr2i18n("A", nullptr));
#if QT_CONFIG(shortcut)
        pbA->setShortcut(tr2i18n("A", nullptr));
#endif // QT_CONFIG(shortcut)
        pbD->setText(tr2i18n("D", nullptr));
#if QT_CONFIG(shortcut)
        pbD->setShortcut(tr2i18n("D", nullptr));
#endif // QT_CONFIG(shortcut)
        pbC1->setText(tr2i18n("C1", nullptr));
#if QT_CONFIG(tooltip)
        pbHyp->setToolTip(tr2i18n("Hyperbolic mode", nullptr));
#endif // QT_CONFIG(tooltip)
        pbHyp->setText(tr2i18n("Hyp", nullptr));
#if QT_CONFIG(shortcut)
        pbHyp->setShortcut(tr2i18n("H", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        pbXOR->setToolTip(tr2i18n("Bitwise XOR", nullptr));
#endif // QT_CONFIG(tooltip)
        pbXOR->setText(tr2i18n("XOR", nullptr));
#if QT_CONFIG(tooltip)
        pbMean->setToolTip(tr2i18n("Mean", nullptr));
#endif // QT_CONFIG(tooltip)
        pbMean->setText(tr2i18n("Mea", nullptr));
#if QT_CONFIG(tooltip)
        pbFactorial->setToolTip(tr2i18n("Factorial", nullptr));
#endif // QT_CONFIG(tooltip)
        pbFactorial->setText(tr2i18n("x!", nullptr));
#if QT_CONFIG(tooltip)
        pbDat->setToolTip(tr2i18n("Enter data", nullptr));
#endif // QT_CONFIG(tooltip)
        pbDat->setText(tr2i18n("Dat", nullptr));
#if QT_CONFIG(shortcut)
        pbDat->setShortcut(tr2i18n("D", nullptr));
#endif // QT_CONFIG(shortcut)
        pbC3->setText(tr2i18n("C3", nullptr));
#if QT_CONFIG(tooltip)
        pbCmp->setToolTip(tr2i18n("One's complement", nullptr));
#endif // QT_CONFIG(tooltip)
        pbCmp->setText(tr2i18n("Cmp", nullptr));
#if QT_CONFIG(tooltip)
        pbCSt->setToolTip(tr2i18n("Clear data store", nullptr));
#endif // QT_CONFIG(tooltip)
        pbCSt->setText(tr2i18n("CSt", nullptr));
#if QT_CONFIG(tooltip)
        pbRsh->setToolTip(tr2i18n("Right bit shift", nullptr));
#endif // QT_CONFIG(tooltip)
        pbRsh->setText(tr2i18n("Rsh", nullptr));
#if QT_CONFIG(tooltip)
        pbCos->setToolTip(tr2i18n("Cosine", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(accessibility)
        pbCos->setAccessibleName(tr2i18n("Cosine", nullptr));
#endif // QT_CONFIG(accessibility)
        pbCos->setText(tr2i18n("Cos", nullptr));
#if QT_CONFIG(shortcut)
        pbCos->setShortcut(tr2i18n("C", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        pbReci->setToolTip(tr2i18n("Reciprocal", nullptr));
#endif // QT_CONFIG(tooltip)
        pbReci->setText(tr2i18n("1/X", nullptr));
#if QT_CONFIG(shortcut)
        pbReci->setShortcut(tr2i18n("R", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        pbLn->setToolTip(tr2i18n("Natural log", nullptr));
#endif // QT_CONFIG(tooltip)
        pbLn->setText(tr2i18n("Ln", nullptr));
#if QT_CONFIG(shortcut)
        pbLn->setShortcut(tr2i18n("N", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        pbI->setToolTip(tr2i18n("Imaginary Unit", nullptr));
#endif // QT_CONFIG(tooltip)
        pbI->setText(tr2i18n("i", nullptr));
#if QT_CONFIG(shortcut)
        pbI->setShortcut(tr2i18n("i", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        pbMed->setToolTip(tr2i18n("Median", nullptr));
#endif // QT_CONFIG(tooltip)
        pbMed->setText(tr2i18n("Med", nullptr));
        pbC5->setText(tr2i18n("C5", nullptr));
        pbC4->setText(tr2i18n("C4", nullptr));
#if QT_CONFIG(tooltip)
        pbSd->setToolTip(tr2i18n("Standard deviation", nullptr));
#endif // QT_CONFIG(tooltip)
        pbSd->setText(tr2i18n("SD", nullptr));
#if QT_CONFIG(tooltip)
        pbLsh->setToolTip(tr2i18n("Left bit shift", nullptr));
#endif // QT_CONFIG(tooltip)
        pbLsh->setText(tr2i18n("Lsh", nullptr));
#if QT_CONFIG(tooltip)
        pbSqrt->setToolTip(tr2i18n("Square root", nullptr));
#endif // QT_CONFIG(tooltip)
        pbSqrt->setText(tr2i18n("sqrt", nullptr));
#if QT_CONFIG(tooltip)
        pbPower->setToolTip(tr2i18n("x to the power of y", nullptr));
#endif // QT_CONFIG(tooltip)
        pbPower->setText(tr2i18n("pow", nullptr));
#if QT_CONFIG(accessibility)
        pb6->setAccessibleName(tr2i18n("Six", nullptr));
#endif // QT_CONFIG(accessibility)
        pb6->setText(tr2i18n("6", nullptr));
#if QT_CONFIG(shortcut)
        pb6->setShortcut(tr2i18n("6", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(accessibility)
        pb2->setAccessibleName(tr2i18n("Two", nullptr));
#endif // QT_CONFIG(accessibility)
        pb2->setText(tr2i18n("2", nullptr));
#if QT_CONFIG(shortcut)
        pb2->setShortcut(tr2i18n("2", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        pbPercent->setToolTip(tr2i18n("Percent", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(accessibility)
        pbPercent->setAccessibleName(tr2i18n("Percentage", nullptr));
#endif // QT_CONFIG(accessibility)
        pbPercent->setText(tr2i18n("%", nullptr));
#if QT_CONFIG(accessibility)
        pb9->setAccessibleName(tr2i18n("Nine", nullptr));
#endif // QT_CONFIG(accessibility)
        pb9->setText(tr2i18n("9", nullptr));
#if QT_CONFIG(shortcut)
        pb9->setShortcut(tr2i18n("9", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(accessibility)
        pb4->setAccessibleName(tr2i18n("Four", nullptr));
#endif // QT_CONFIG(accessibility)
        pb4->setText(tr2i18n("4", nullptr));
#if QT_CONFIG(shortcut)
        pb4->setShortcut(tr2i18n("4", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(accessibility)
        pb5->setAccessibleName(tr2i18n("Five", nullptr));
#endif // QT_CONFIG(accessibility)
        pb5->setText(tr2i18n("5", nullptr));
#if QT_CONFIG(shortcut)
        pb5->setShortcut(tr2i18n("5", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        pbMinus->setToolTip(tr2i18n("Minus", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(accessibility)
        pbMinus->setAccessibleName(tr2i18n("Subtract", nullptr));
#endif // QT_CONFIG(accessibility)
        pbMinus->setText(tr2i18n("\342\210\222", "- calculator button"));
#if QT_CONFIG(tooltip)
        pbPlus->setToolTip(tr2i18n("Plus", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(accessibility)
        pbPlus->setAccessibleName(tr2i18n("Add", nullptr));
#endif // QT_CONFIG(accessibility)
        pbPlus->setText(tr2i18n("+", "+ calculator button"));
#if QT_CONFIG(accessibility)
        pb1->setAccessibleName(tr2i18n("One", nullptr));
#endif // QT_CONFIG(accessibility)
        pb1->setText(tr2i18n("1", nullptr));
#if QT_CONFIG(shortcut)
        pb1->setShortcut(tr2i18n("1", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        pbDivision->setToolTip(tr2i18n("Division", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(accessibility)
        pbDivision->setAccessibleName(tr2i18n("Divide", nullptr));
#endif // QT_CONFIG(accessibility)
        pbDivision->setText(tr2i18n("\303\267", nullptr));
#if QT_CONFIG(tooltip)
        pbMultiplication->setToolTip(tr2i18n("Multiplication", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(accessibility)
        pbMultiplication->setAccessibleName(tr2i18n("Multiply", nullptr));
#endif // QT_CONFIG(accessibility)
        pbMultiplication->setText(tr2i18n("\303\227", nullptr));
#if QT_CONFIG(accessibility)
        pb0->setAccessibleName(tr2i18n("Zero", nullptr));
#endif // QT_CONFIG(accessibility)
        pb0->setText(tr2i18n("0", nullptr));
#if QT_CONFIG(shortcut)
        pb0->setShortcut(tr2i18n("0", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        pbPeriod->setToolTip(tr2i18n("Decimal point", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(accessibility)
        pbPeriod->setAccessibleName(tr2i18n("Decimal point", nullptr));
#endif // QT_CONFIG(accessibility)
        pbPeriod->setText(tr2i18n(".", ". calculator button"));
#if QT_CONFIG(accessibility)
        pb3->setAccessibleName(tr2i18n("Three", nullptr));
#endif // QT_CONFIG(accessibility)
        pb3->setText(tr2i18n("3", nullptr));
#if QT_CONFIG(shortcut)
        pb3->setShortcut(tr2i18n("3", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        pbEqual->setToolTip(tr2i18n("Result", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(accessibility)
        pbEqual->setAccessibleName(tr2i18n("Equals", nullptr));
#endif // QT_CONFIG(accessibility)
        pbEqual->setText(tr2i18n("=", "= calculator button"));
#if QT_CONFIG(accessibility)
        pb7->setAccessibleName(tr2i18n("Seven", nullptr));
#endif // QT_CONFIG(accessibility)
        pb7->setText(tr2i18n("7", nullptr));
#if QT_CONFIG(shortcut)
        pb7->setShortcut(tr2i18n("7", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(accessibility)
        pb8->setAccessibleName(tr2i18n("Eight", nullptr));
#endif // QT_CONFIG(accessibility)
        pb8->setText(tr2i18n("8", nullptr));
#if QT_CONFIG(shortcut)
        pb8->setShortcut(tr2i18n("8", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        pbAllClear->setToolTip(tr2i18n("Clear all", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(accessibility)
        pbAllClear->setAccessibleName(tr2i18n("All clear", nullptr));
#endif // QT_CONFIG(accessibility)
        pbAllClear->setText(tr2i18n("AC", nullptr));
#if QT_CONFIG(tooltip)
        pbMemClear->setToolTip(tr2i18n("Memory clear", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(accessibility)
        pbMemClear->setAccessibleName(tr2i18n("Memory clear", nullptr));
#endif // QT_CONFIG(accessibility)
        pbMemClear->setText(tr2i18n("MC", nullptr));
#if QT_CONFIG(tooltip)
        pbMemRecall->setToolTip(tr2i18n("Memory recall", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(accessibility)
        pbMemRecall->setAccessibleName(tr2i18n("Memory recall", nullptr));
#endif // QT_CONFIG(accessibility)
        pbMemRecall->setText(tr2i18n("MR", nullptr));
#if QT_CONFIG(tooltip)
        pbPlusMinus->setToolTip(tr2i18n("Change sign", nullptr));
#endif // QT_CONFIG(tooltip)
        pbPlusMinus->setText(tr2i18n("+/\342\210\222", nullptr));
#if QT_CONFIG(tooltip)
        pbParenOpen->setToolTip(tr2i18n("Open parenthesis", nullptr));
#endif // QT_CONFIG(tooltip)
        pbParenOpen->setText(tr2i18n("(", "( calculator button"));
#if QT_CONFIG(tooltip)
        pbClear->setToolTip(tr2i18n("Clear", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(accessibility)
        pbClear->setAccessibleName(tr2i18n("Clear", nullptr));
#endif // QT_CONFIG(accessibility)
        pbClear->setText(tr2i18n("C", nullptr));
#if QT_CONFIG(tooltip)
        pbMemPlusMinus->setToolTip(tr2i18n("Add to memory", nullptr));
#endif // QT_CONFIG(tooltip)
        pbMemPlusMinus->setText(tr2i18n("M+", nullptr));
#if QT_CONFIG(tooltip)
        pbMemStore->setToolTip(tr2i18n("Memory store", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(accessibility)
        pbMemStore->setAccessibleName(tr2i18n("Memory store", nullptr));
#endif // QT_CONFIG(accessibility)
        pbMemStore->setText(tr2i18n("MS", nullptr));
#if QT_CONFIG(tooltip)
        pbParenClose->setToolTip(tr2i18n("Close parenthesis", nullptr));
#endif // QT_CONFIG(tooltip)
        pbParenClose->setText(tr2i18n(")", ") calculator button"));
#if QT_CONFIG(accessibility)
        pbBackspace->setAccessibleName(tr2i18n("Return", nullptr));
#endif // QT_CONFIG(accessibility)
        pbBackspace->setText(tr2i18n("\342\206\220", nullptr));
#if QT_CONFIG(shortcut)
        pbBackspace->setShortcut(tr2i18n("Backspace", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        pbShift->setToolTip(tr2i18n("Second button function", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(accessibility)
        pbShift->setAccessibleName(tr2i18n("Shift", nullptr));
#endif // QT_CONFIG(accessibility)
        pbShift->setText(tr2i18n("Shift", nullptr));
#if QT_CONFIG(shortcut)
        pbShift->setShortcut(tr2i18n("Ctrl+2", nullptr));
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class KCalculator: public Ui_KCalculator {};
} // namespace Ui

QT_END_NAMESPACE

#endif // KCALC_H

