/********************************************************************************
** Form generated from reading UI file 'general.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GENERAL_H
#define UI_GENERAL_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>

QT_BEGIN_NAMESPACE

class Ui_General
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *precisionGroupBox;
    QHBoxLayout *hboxLayout;
    QGridLayout *gridLayout_2;
    QLabel *textLabel1;
    QSpinBox *kcfg_Precision;
    QCheckBox *kcfg_Fixed;
    QSpinBox *kcfg_FixedPrecision;
    QSpacerItem *spacerItem;
    QGroupBox *groupBox;
    QVBoxLayout *_2;
    QHBoxLayout *horizontalLayout;
    QCheckBox *kcfg_GroupDigits;
    QGridLayout *gridLayout;
    QLabel *grouping_binary_lb;
    QLabel *grouping_octal_lb;
    QLabel *grouping_hexadecimal_lb;
    QSpinBox *kcfg_BinaryGrouping;
    QSpinBox *kcfg_OctalGrouping;
    QSpinBox *kcfg_HexadecimalGrouping;
    QGroupBox *miscGroupBox;
    QVBoxLayout *vboxLayout1;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *kcfg_Beep;
    QCheckBox *kcfg_CaptionResult;
    QCheckBox *kcfg_TwosComplement;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *General)
    {
        if (General->objectName().isEmpty())
            General->setObjectName("General");
        General->resize(396, 592);
        vboxLayout = new QVBoxLayout(General);
        vboxLayout->setObjectName("vboxLayout");
        precisionGroupBox = new QGroupBox(General);
        precisionGroupBox->setObjectName("precisionGroupBox");
        hboxLayout = new QHBoxLayout(precisionGroupBox);
        hboxLayout->setObjectName("hboxLayout");
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName("gridLayout_2");
        textLabel1 = new QLabel(precisionGroupBox);
        textLabel1->setObjectName("textLabel1");
        textLabel1->setWordWrap(false);

        gridLayout_2->addWidget(textLabel1, 0, 0, 1, 1);

        kcfg_Precision = new QSpinBox(precisionGroupBox);
        kcfg_Precision->setObjectName("kcfg_Precision");

        gridLayout_2->addWidget(kcfg_Precision, 0, 1, 1, 1);

        kcfg_Fixed = new QCheckBox(precisionGroupBox);
        kcfg_Fixed->setObjectName("kcfg_Fixed");

        gridLayout_2->addWidget(kcfg_Fixed, 1, 0, 1, 1);

        kcfg_FixedPrecision = new QSpinBox(precisionGroupBox);
        kcfg_FixedPrecision->setObjectName("kcfg_FixedPrecision");
        kcfg_FixedPrecision->setEnabled(false);

        gridLayout_2->addWidget(kcfg_FixedPrecision, 1, 1, 1, 1);


        hboxLayout->addLayout(gridLayout_2);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Minimum);

        hboxLayout->addItem(spacerItem);


        vboxLayout->addWidget(precisionGroupBox);

        groupBox = new QGroupBox(General);
        groupBox->setObjectName("groupBox");
        _2 = new QVBoxLayout(groupBox);
        _2->setObjectName("_2");
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName("horizontalLayout");
        kcfg_GroupDigits = new QCheckBox(groupBox);
        kcfg_GroupDigits->setObjectName("kcfg_GroupDigits");
        kcfg_GroupDigits->setChecked(true);

        horizontalLayout->addWidget(kcfg_GroupDigits);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName("gridLayout");
        grouping_binary_lb = new QLabel(groupBox);
        grouping_binary_lb->setObjectName("grouping_binary_lb");
        grouping_binary_lb->setEnabled(true);

        gridLayout->addWidget(grouping_binary_lb, 0, 0, 1, 1);

        grouping_octal_lb = new QLabel(groupBox);
        grouping_octal_lb->setObjectName("grouping_octal_lb");

        gridLayout->addWidget(grouping_octal_lb, 1, 0, 1, 1);

        grouping_hexadecimal_lb = new QLabel(groupBox);
        grouping_hexadecimal_lb->setObjectName("grouping_hexadecimal_lb");

        gridLayout->addWidget(grouping_hexadecimal_lb, 2, 0, 1, 1);

        kcfg_BinaryGrouping = new QSpinBox(groupBox);
        kcfg_BinaryGrouping->setObjectName("kcfg_BinaryGrouping");

        gridLayout->addWidget(kcfg_BinaryGrouping, 0, 1, 1, 1);

        kcfg_OctalGrouping = new QSpinBox(groupBox);
        kcfg_OctalGrouping->setObjectName("kcfg_OctalGrouping");

        gridLayout->addWidget(kcfg_OctalGrouping, 1, 1, 1, 1);

        kcfg_HexadecimalGrouping = new QSpinBox(groupBox);
        kcfg_HexadecimalGrouping->setObjectName("kcfg_HexadecimalGrouping");

        gridLayout->addWidget(kcfg_HexadecimalGrouping, 2, 1, 1, 1);


        horizontalLayout->addLayout(gridLayout);


        _2->addLayout(horizontalLayout);


        vboxLayout->addWidget(groupBox);

        miscGroupBox = new QGroupBox(General);
        miscGroupBox->setObjectName("miscGroupBox");
        vboxLayout1 = new QVBoxLayout(miscGroupBox);
        vboxLayout1->setObjectName("vboxLayout1");
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName("verticalLayout_2");
        kcfg_Beep = new QCheckBox(miscGroupBox);
        kcfg_Beep->setObjectName("kcfg_Beep");
        kcfg_Beep->setChecked(true);

        verticalLayout_2->addWidget(kcfg_Beep);

        kcfg_CaptionResult = new QCheckBox(miscGroupBox);
        kcfg_CaptionResult->setObjectName("kcfg_CaptionResult");

        verticalLayout_2->addWidget(kcfg_CaptionResult);

        kcfg_TwosComplement = new QCheckBox(miscGroupBox);
        kcfg_TwosComplement->setObjectName("kcfg_TwosComplement");
        kcfg_TwosComplement->setChecked(true);

        verticalLayout_2->addWidget(kcfg_TwosComplement);


        vboxLayout1->addLayout(verticalLayout_2);


        vboxLayout->addWidget(miscGroupBox);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        vboxLayout->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        textLabel1->setBuddy(kcfg_Precision);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(kcfg_Precision, kcfg_Fixed);
        QWidget::setTabOrder(kcfg_Fixed, kcfg_FixedPrecision);

        retranslateUi(General);
        QObject::connect(kcfg_Fixed, &QCheckBox::toggled, kcfg_FixedPrecision, &QSpinBox::setEnabled);

        QMetaObject::connectSlotsByName(General);
    } // setupUi

    void retranslateUi(QWidget *General)
    {
        General->setWindowTitle(tr2i18n("General", nullptr));
        precisionGroupBox->setTitle(tr2i18n("Precision", nullptr));
        textLabel1->setText(tr2i18n("&Maximum number of digits:", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_Precision->setToolTip(tr2i18n("Maximum number of digits displayed", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        kcfg_Precision->setWhatsThis(tr2i18n("KCalc can compute with many more digits than the number that fits on the display. This setting gives the maximum number of digits displayed, before KCalc starts using scientific notation, i.e. notation of the type 2.34e12.", nullptr));
#endif // QT_CONFIG(whatsthis)
#if QT_CONFIG(tooltip)
        kcfg_Fixed->setToolTip(tr2i18n("Whether to use fixed decimal places", nullptr));
#endif // QT_CONFIG(tooltip)
        kcfg_Fixed->setText(tr2i18n("Set &decimal precision", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_FixedPrecision->setToolTip(tr2i18n("Number of fixed decimal digits", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox->setTitle(tr2i18n("Grouping", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_GroupDigits->setToolTip(tr2i18n("Whether to group digits", nullptr));
#endif // QT_CONFIG(tooltip)
        kcfg_GroupDigits->setText(tr2i18n("Group digits", nullptr));
        grouping_binary_lb->setText(tr2i18n("Binary", nullptr));
        grouping_octal_lb->setText(tr2i18n("Octal", nullptr));
        grouping_hexadecimal_lb->setText(tr2i18n("Hexadecimal", nullptr));
        miscGroupBox->setTitle(tr2i18n("Misc", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_Beep->setToolTip(tr2i18n("Whether to beep on error", nullptr));
#endif // QT_CONFIG(tooltip)
        kcfg_Beep->setText(tr2i18n("&Beep on error", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_CaptionResult->setToolTip(tr2i18n("Whether to show the result in the window title", nullptr));
#endif // QT_CONFIG(tooltip)
        kcfg_CaptionResult->setText(tr2i18n("Show &result in window title", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_TwosComplement->setToolTip(tr2i18n("Whether to use Two's Complement for non-decimal numbers", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(whatsthis)
        kcfg_TwosComplement->setWhatsThis(tr2i18n("Select to use Two's Complement notation for Binary, Octal and Hexidecimal numbers. This is a common notation to represent negative numbers for non-decimal numbers in computers.", nullptr));
#endif // QT_CONFIG(whatsthis)
        kcfg_TwosComplement->setText(tr2i18n("Two's complement", nullptr));
    } // retranslateUi

};

namespace Ui {
    class General: public Ui_General {};
} // namespace Ui

QT_END_NAMESPACE

#endif // GENERAL_H

