/********************************************************************************
** Form generated from reading UI file 'fonts.ui'
**
** Created by: Qt User Interface Compiler version 6.11.0
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FONTS_H
#define UI_FONTS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <klocalizedstring.h>
#include "kfontrequester.h"

QT_BEGIN_NAMESPACE

class Ui_Fonts
{
public:
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout;
    QLabel *buttonlabel;
    KFontRequester *kcfg_ButtonFont;
    QLabel *displaylabel;
    KFontRequester *kcfg_DisplayFont;
    QLabel *historylabel;
    KFontRequester *kcfg_HistoryFont;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *Fonts)
    {
        if (Fonts->objectName().isEmpty())
            Fonts->setObjectName("Fonts");
        Fonts->resize(288, 74);
        verticalLayout = new QVBoxLayout(Fonts);
        verticalLayout->setObjectName("verticalLayout");
        formLayout = new QFormLayout();
        formLayout->setObjectName("formLayout");
        formLayout->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        buttonlabel = new QLabel(Fonts);
        buttonlabel->setObjectName("buttonlabel");

        formLayout->setWidget(0, QFormLayout::ItemRole::LabelRole, buttonlabel);

        kcfg_ButtonFont = new KFontRequester(Fonts);
        kcfg_ButtonFont->setObjectName("kcfg_ButtonFont");
        kcfg_ButtonFont->setFocusPolicy(Qt::WheelFocus);

        formLayout->setWidget(0, QFormLayout::ItemRole::FieldRole, kcfg_ButtonFont);

        displaylabel = new QLabel(Fonts);
        displaylabel->setObjectName("displaylabel");

        formLayout->setWidget(1, QFormLayout::ItemRole::LabelRole, displaylabel);

        kcfg_DisplayFont = new KFontRequester(Fonts);
        kcfg_DisplayFont->setObjectName("kcfg_DisplayFont");
        kcfg_DisplayFont->setFocusPolicy(Qt::WheelFocus);

        formLayout->setWidget(1, QFormLayout::ItemRole::FieldRole, kcfg_DisplayFont);

        historylabel = new QLabel(Fonts);
        historylabel->setObjectName("historylabel");

        formLayout->setWidget(2, QFormLayout::ItemRole::LabelRole, historylabel);

        kcfg_HistoryFont = new KFontRequester(Fonts);
        kcfg_HistoryFont->setObjectName("kcfg_HistoryFont");
        kcfg_HistoryFont->setFocusPolicy(Qt::WheelFocus);

        formLayout->setWidget(2, QFormLayout::ItemRole::FieldRole, kcfg_HistoryFont);


        verticalLayout->addLayout(formLayout);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        verticalLayout->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        buttonlabel->setBuddy(kcfg_ButtonFont);
        displaylabel->setBuddy(kcfg_DisplayFont);
        historylabel->setBuddy(kcfg_HistoryFont);
#endif // QT_CONFIG(shortcut)

        retranslateUi(Fonts);

        QMetaObject::connectSlotsByName(Fonts);
    } // setupUi

    void retranslateUi(QWidget *Fonts)
    {
        buttonlabel->setText(tr2i18n("&Button font:", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_ButtonFont->setToolTip(tr2i18n("The font to use for the buttons", nullptr));
#endif // QT_CONFIG(tooltip)
        displaylabel->setText(tr2i18n("&Display font:", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_DisplayFont->setToolTip(tr2i18n("The font to use in the display", nullptr));
#endif // QT_CONFIG(tooltip)
        historylabel->setText(tr2i18n("&History font:", nullptr));
#if QT_CONFIG(tooltip)
        kcfg_HistoryFont->setToolTip(tr2i18n("The font to use in the history", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)Fonts;
    } // retranslateUi

};

namespace Ui {
    class Fonts: public Ui_Fonts {};
} // namespace Ui

QT_END_NAMESPACE

#endif // FONTS_H

